import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.multi_segment_packer import (
    MultiSegmentPacker,
)
from keras_hub.src.models.causal_lm_preprocessor import CausalLMPreprocessor
from keras_hub.src.models.pali_gemma.pali_gemma_backbone import (
    PaliGemmaBackbone,
)
from keras_hub.src.models.pali_gemma.pali_gemma_image_converter import (
    PaliGemmaImageConverter,
)
from keras_hub.src.models.pali_gemma.pali_gemma_tokenizer import (
    PaliGemmaTokenizer,
)
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export("keras_hub.models.PaliGemmaCausalLMPreprocessor")
class PaliGemmaCausalLMPreprocessor(CausalLMPreprocessor):
    backbone_cls = PaliGemmaBackbone
    tokenizer_cls = PaliGemmaTokenizer
    image_converter_cls = PaliGemmaImageConverter

    def __init__(
        self,
        tokenizer,
        image_converter=None,
        sequence_length=1024,
        add_start_token=True,
        add_end_token=True,
        **kwargs,
    ):
        super().__init__(
            tokenizer=tokenizer,
            sequence_length=sequence_length,
            add_start_token=add_start_token,
            add_end_token=add_end_token,
            **kwargs,
        )
        self.image_converter = image_converter

    def build(self, input_shape):
        # Defer packer creation to `build()` so that we can be sure tokenizer
        # assets have loaded when restoring a saved model.
        self.packer = MultiSegmentPacker(
            start_value=self.tokenizer.start_token_id,
            end_value=self.tokenizer.end_token_id,
            pad_value=self.tokenizer.pad_token_id,
            sep_value=[],
            sequence_length=self.sequence_length,
        )
        self.built = True

    @preprocessing_function
    def call(
        self,
        x,
        y=None,
        sample_weight=None,
        sequence_length=None,
    ):
        sequence_length = sequence_length or self.sequence_length
        images, prompts, responses = x["images"], x["prompts"], x["responses"]
        prompts = self.tokenizer(prompts)
        responses = self.tokenizer(responses)
        if self.image_converter:
            images = self.image_converter(images)
        # Pad with one extra token to account for the truncation below.
        token_ids, segment_ids = self.packer(
            (prompts, responses),
            sequence_length=sequence_length + 1,
            add_start_value=self.add_start_token,
            add_end_value=self.add_end_token,
        )
        padding_mask = token_ids != self.tokenizer.pad_token_id
        response_mask = segment_ids == 1
        # The last token does not have a next token, so we truncate it out.
        x = {
            "token_ids": token_ids[..., :-1],
            "response_mask": response_mask[..., :-1],
            "padding_mask": padding_mask[..., :-1],
            "images": images,
        }
        # Target `y` will be the next token.
        y = token_ids[..., 1:]
        # Only compute the loss for labels in the response.
        sample_weight = response_mask[..., 1:]
        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)

    @preprocessing_function
    def generate_preprocess(
        self,
        x,
        sequence_length=None,
    ):
        """Convert strings to integer token input for generation.

        Similar to calling the layer for training, this method takes in strings
        or tensor strings, tokenizes and packs the input, and computes a padding
        mask masking all inputs not filled in with a padded value.

        Unlike calling the layer for training, this method does not compute
        labels and will never append a `tokenizer.end_token_id` to the end of
        the sequence (as generation is expected to continue at the end of the
        inputted prompt).
        """
        if not self.built:
            self.build(None)
        sequence_length = sequence_length or self.sequence_length

        images, prompts = x["images"], x["prompts"]
        prompts = self.tokenizer(prompts)
        if self.image_converter:
            images = self.image_converter(images)
        if "responses" in x:
            responses = self.tokenizer(x["responses"])
            segments = (prompts, responses)
        else:
            segments = (prompts,)
        token_ids, segment_ids = self.packer(
            segments,
            sequence_length=sequence_length,
            add_end_value=False,
        )
        padding_mask = token_ids != self.tokenizer.pad_token_id
        response_mask = segment_ids == 1
        return {
            "images": images,
            "token_ids": token_ids,
            "response_mask": response_mask,
            "padding_mask": padding_mask,
        }



"""PaliGemma model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "pali_gemma_3b_mix_224": {
        "metadata": {
            "description": (
                "image size 224, mix fine tuned, text sequence length is 256"
            ),
            "params": 2923335408,
            "path": "pali_gemma",
        },
        "kaggle_handle": "kaggle://keras/paligemma/keras/pali_gemma_3b_mix_224/4",
    },
    "pali_gemma_3b_mix_448": {
        "metadata": {
            "description": (
                "image size 448, mix fine tuned, text sequence length is 512"
            ),
            "params": 2924220144,
            "path": "pali_gemma",
        },
        "kaggle_handle": "kaggle://keras/paligemma/keras/pali_gemma_3b_mix_448/4",
    },
    "pali_gemma_3b_224": {
        "metadata": {
            "description": (
                "image size 224, pre trained, text sequence length is 128"
            ),
            "params": 2923335408,
            "path": "pali_gemma",
        },
        "kaggle_handle": "kaggle://keras/paligemma/keras/pali_gemma_3b_224/4",
    },
    "pali_gemma_3b_448": {
        "metadata": {
            "description": (
                "image size 448, pre trained, text sequence length is 512"
            ),
            "params": 2924220144,
            "path": "pali_gemma",
        },
        "kaggle_handle": "kaggle://keras/paligemma/keras/pali_gemma_3b_448/4",
    },
    "pali_gemma_3b_896": {
        "metadata": {
            "description": (
                "image size 896, pre trained, text sequence length is 512"
            ),
            "params": 2927759088,
            "path": "pali_gemma",
        },
        "kaggle_handle": "kaggle://keras/paligemma/keras/pali_gemma_3b_896/4",
    },
    # PaliGemma2
    "pali_gemma_2_ft_docci_3b_448": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been fine-tuned on the DOCCI dataset "
                "for improved descriptions with fine-grained details."
            ),
            "params": 3032979696,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma_2_ft_docci_3b_448/2",
    },
    "pali_gemma2_ft_docci_10b_448": {
        "metadata": {
            "description": (
                "10 billion parameter, 27-layer for SigLIP-So400m vision "
                "encoder and 42-layer Gemma2 9B lanuage model. This model has "
                "been fine-tuned on the DOCCI dataset for improved "
                "descriptions with fine-grained details."
            ),
            "params": 9663294192,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_ft_docci_10b_448/2",
    },
    "pali_gemma2_mix_3b_224": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 3032094960,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_mix_3b_224/2",
    },
    "pali_gemma2_mix_3b_448": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 3032979696,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_mix_3b_448/2",
    },
    "pali_gemma2_mix_10b_224": {
        "metadata": {
            "description": (
                "10 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 42-layer Gemma2 9B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 9662409456,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_mix_10b_224/2",
    },
    "pali_gemma2_mix_10b_448": {
        "metadata": {
            "description": (
                "10 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 42-layer Gemma2 9B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 9663294192,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_mix_10b_448/2",
    },
    "pali_gemma2_mix_28b_224": {
        "metadata": {
            "description": (
                "28 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 46-layer Gemma2 27B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 27650192112,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_28b_mix_224/2",
    },
    "pali_gemma2_mix_28b_448": {
        "metadata": {
            "description": (
                "28 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 46-layer Gemma2 27B lanuage "
                "model. This model has been fine-tuned on a wide range of "
                "vision-language tasks and domains."
            ),
            "params": 27650192112,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_28b_mix_448/2",
    },
    "pali_gemma2_pt_3b_224": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 3032094960,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_3b_224/2",
    },
    "pali_gemma2_pt_3b_448": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 3032979696,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_3b_448/2",
    },
    "pali_gemma2_pt_3b_896": {
        "metadata": {
            "description": (
                "3 billion parameter, image size 896, 27-layer for "
                "SigLIP-So400m vision encoder and 26-layer Gemma2 2B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 3036518640,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_3b_896/2",
    },
    "pali_gemma2_pt_10b_224": {
        "metadata": {
            "description": (
                "10 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 42-layer Gemma2 9B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 9662409456,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_10b_224/2",
    },
    "pali_gemma2_pt_10b_448": {
        "metadata": {
            "description": (
                "10 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 42-layer Gemma2 9B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 9663294192,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_10b_448/2",
    },
    "pali_gemma2_pt_10b_896": {
        "metadata": {
            "description": (
                "10 billion parameter, image size 896, 27-layer for "
                "SigLIP-So400m vision encoder and 42-layer Gemma2 9B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 9666833136,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_10b_896/2",
    },
    "pali_gemma2_pt_28b_224": {
        "metadata": {
            "description": (
                "28 billion parameter, image size 224, 27-layer for "
                "SigLIP-So400m vision encoder and 46-layer Gemma2 27B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 27650192112,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_28b_224/3",
    },
    "pali_gemma2_pt_28b_448": {
        "metadata": {
            "description": (
                "28 billion parameter, image size 448, 27-layer for "
                "SigLIP-So400m vision encoder and 46-layer Gemma2 27B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 27650192112,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_28b_448/2",
    },
    "pali_gemma2_pt_28b_896": {
        "metadata": {
            "description": (
                "28 billion parameter, image size 896, 27-layer for "
                "SigLIP-So400m vision encoder and 46-layer Gemma2 27B lanuage "
                "model. This model has been pre-trained on a mixture of "
                "datasets."
            ),
            "params": 27650192112,
            "official_name": "PaliGemma2",
            "path": "pali_gemma2",
            "model_card": "https://www.kaggle.com/models/google/paligemma-2",
        },
        "kaggle_handle": "kaggle://keras/paligemma2/keras/pali_gemma2_pt_28b_896/2",
    },
}



import keras
from keras import ops

from keras_hub.src.api_export import keras_hub_export


@keras_hub_export("keras_hub.layers.RMSNormalization")
class RMSNormalization(keras.layers.Layer):
    """Root Mean Square (RMS) Normalization layer.

    This layer normalizes the input tensor based on its RMS value and applies
    a learned scaling factor.

    Args:
        input_dim: int. The dimensionality of the input tensor.
    """

    def __init__(self, input_dim):
        super().__init__()
        self.scale = self.add_weight(
            name="scale", shape=(input_dim,), initializer="ones"
        )

    def call(self, x):
        """Applies RMS normalization to the input tensor.

        Args:
            x: Input tensor of shape (batch_size, input_dim).

        Returns:
            The RMS-normalized tensor of the same shape (batch_size, input_dim),
            scaled by the learned `scale` parameter.
        """
        x = ops.cast(x, float)
        rrms = ops.rsqrt(ops.mean(ops.square(x), axis=-1, keepdims=True) + 1e-6)
        return (x * rrms) * self.scale



from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.inpaint import Inpaint
from keras_hub.src.models.stable_diffusion_3.stable_diffusion_3_backbone import (  # noqa: E501
    StableDiffusion3Backbone,
)
from keras_hub.src.models.stable_diffusion_3.stable_diffusion_3_text_to_image_preprocessor import (  # noqa: E501
    StableDiffusion3TextToImagePreprocessor,
)


@keras_hub_export("keras_hub.models.StableDiffusion3Inpaint")
class StableDiffusion3Inpaint(Inpaint):
    """An end-to-end Stable Diffusion 3 model for inpaint generation.

    This model has a `generate()` method, which generates images based
    on a combination of a reference image, mask and a text prompt.

    Args:
        backbone: A `keras_hub.models.StableDiffusion3Backbone` instance.
        preprocessor: A
            `keras_hub.models.StableDiffusion3TextToImagePreprocessor` instance.

    Examples:

    Use `generate()` to do image generation.
    ```python
    reference_image = np.ones((1024, 1024, 3), dtype="float32")
    reference_mask = np.ones((1024, 1024), dtype="float32")
    inpaint = keras_hub.models.StableDiffusion3Inpaint.from_preset(
        "stable_diffusion_3_medium", image_shape=(512, 512, 3)
    )
    inpaint.generate(
        reference_image,
        reference_mask,
        "Astronaut in a jungle, cold color palette, muted colors, detailed, 8k",
    )

    # Generate with batched prompts.
    reference_images = np.ones((2, 512, 512, 3), dtype="float32")
    reference_mask = np.ones((2, 1024, 1024), dtype="float32")
    inpaint.generate(
        reference_images,
        reference_mask,
        ["cute wallpaper art of a cat", "cute wallpaper art of a dog"]
    )

    # Generate with different `num_steps`, `guidance_scale` and `strength`.
    inpaint.generate(
        reference_image,
        reference_mask,
        "Astronaut in a jungle, cold color palette, muted colors, detailed, 8k",
        num_steps=50,
        guidance_scale=5.0,
        strength=0.6,
    )
    ```
    """

    backbone_cls = StableDiffusion3Backbone
    preprocessor_cls = StableDiffusion3TextToImagePreprocessor

    def __init__(
        self,
        backbone,
        preprocessor,
        **kwargs,
    ):
        # === Layers ===
        self.backbone = backbone
        self.preprocessor = preprocessor

        # === Functional Model ===
        inputs = backbone.input
        outputs = backbone.output
        super().__init__(
            inputs=inputs,
            outputs=outputs,
            **kwargs,
        )

    def fit(self, *args, **kwargs):
        raise NotImplementedError(
            "Currently, `fit` is not supported for `StableDiffusion3Inpaint`."
        )

    def generate_step(
        self,
        images,
        masks,
        noises,
        token_ids,
        starting_step,
        num_steps,
        guidance_scale,
    ):
        """A compilable generation function for batched of inputs.

        This function represents the inner, XLA-compilable, generation function
        for batched inputs.

        Args:
            images: A (batch_size, image_height, image_width, 3) tensor
                containing the reference images.
            masks: A (batch_size, image_height, image_width) tensor
                containing the reference masks.
            noises: A (batch_size, latent_height, latent_width, channels) tensor
                containing the noises to be added to the latents. Typically,
                this tensor is sampled from the Gaussian distribution.
            token_ids: A pair of (batch_size, num_tokens) tensor containing the
                tokens based on the input prompts and negative prompts.
            starting_step: int. The number of the starting diffusion step.
            num_steps: int. The number of diffusion steps to take.
            guidance_scale: float. The classifier free guidance scale defined in
                [Classifier-Free Diffusion Guidance](
                https://arxiv.org/abs/2207.12598). Higher scale encourages to
                generate images that are closely linked to prompts, usually at
                the expense of lower image quality.
        """
        token_ids, negative_token_ids = token_ids

        # Get masked images.
        masks = ops.cast(ops.expand_dims(masks, axis=-1) > 0.5, images.dtype)
        masks_latent_size = ops.image.resize(
            masks,
            (self.backbone.latent_shape[1], self.backbone.latent_shape[2]),
            interpolation="nearest",
        )

        # Encode images.
        image_latents = self.backbone.encode_image_step(images)

        # Add noises to latents.
        latents = self.backbone.add_noise_step(
            image_latents, noises, starting_step, num_steps
        )

        # Encode inputs.
        embeddings = self.backbone.encode_text_step(
            token_ids, negative_token_ids
        )

        # Denoise.
        def body_fun(step, latents):
            latents = self.backbone.denoise_step(
                latents,
                embeddings,
                step,
                num_steps,
                guidance_scale,
            )

            # Compute the previous latents x_t -> x_t-1.
            def true_fn():
                next_step = ops.add(step, 1)
                return self.backbone.add_noise_step(
                    image_latents, noises, next_step, num_steps
                )

            init_latents = ops.cond(
                step < ops.subtract(num_steps, 1),
                true_fn,
                lambda: ops.cast(image_latents, noises.dtype),
            )
            latents = ops.add(
                ops.multiply(
                    ops.subtract(1.0, masks_latent_size), init_latents
                ),
                ops.multiply(masks_latent_size, latents),
            )
            return latents

        latents = ops.fori_loop(starting_step, num_steps, body_fun, latents)

        # Decode.
        return self.backbone.decode_step(latents)

    def generate(
        self,
        inputs,
        num_steps=50,
        strength=0.6,
        guidance_scale=7.0,
        seed=None,
    ):
        return super().generate(
            inputs,
            num_steps=num_steps,
            strength=strength,
            guidance_scale=guidance_scale,
            seed=seed,
        )



import keras

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.image_segmenter_preprocessor import (
    ImageSegmenterPreprocessor,
)
from keras_hub.src.models.segformer.segformer_backbone import SegFormerBackbone
from keras_hub.src.models.segformer.segformer_image_converter import (
    SegFormerImageConverter,
)
from keras_hub.src.utils.tensor_utils import preprocessing_function

IMAGENET_DEFAULT_MEAN = [0.485, 0.456, 0.406]
IMAGENET_DEFAULT_STD = [0.229, 0.224, 0.225]


@keras_hub_export("keras_hub.models.SegFormerImageSegmenterPreprocessor")
class SegFormerImageSegmenterPreprocessor(ImageSegmenterPreprocessor):
    backbone_cls = SegFormerBackbone
    image_converter_cls = SegFormerImageConverter

    @preprocessing_function
    def call(self, x, y=None, sample_weight=None):
        if self.image_converter:
            x = self.image_converter(x)
            if y is not None:
                y = self.image_converter(y)

        x = x / 255
        x = (x - IMAGENET_DEFAULT_MEAN) / IMAGENET_DEFAULT_STD

        return keras.utils.pack_x_y_sample_weight(x, y, sample_weight)



from keras_hub.src.models.albert.albert_backbone import AlbertBackbone
from keras_hub.src.models.albert.albert_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, AlbertBackbone)



import keras
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.reversible_embedding import (
    ReversibleEmbedding,
)
from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.llama.llama_decoder import LlamaTransformerDecoder
from keras_hub.src.models.llama.llama_layernorm import LlamaLayerNorm


def _llama_kernel_initializer(stddev=0.02):
    return keras.initializers.RandomNormal(stddev=stddev)


@keras_hub_export("keras_hub.models.LlamaBackbone")
class LlamaBackbone(Backbone):
    """
    The Llama Transformer core architecture with hyperparameters.

    This network implements a Transformer-based decoder network,
    Llama, as described in
    ["Llama 7B"](https://arxiv.org/pdf/2310.06825.pdf).
    It includes the embedding lookups and transformer layers.

    The default constructor gives a fully customizable, randomly initialized
    Llama model with any number of layers, heads, and embedding
    dimensions. To load preset architectures and weights, use the `from_preset`
    constructor.

    Args:
        vocabulary_size (int): The size of the token vocabulary.
        num_layers (int): The number of transformer layers.
        num_query_heads (int): The number of query attention heads for
            each transformer.
        hidden_dim (int): The size of the transformer encoding and pooling
            layers.
        intermediate_dim (int): The output dimension of the first Dense layer in
            a three-layer feedforward network for each transformer.
        num_key_value_heads (int): The number of key and value attention heads
            for each transformer.
        rope_max_wavelength (int, optional): The maximum angular wavelength of
            the sine/cosine curves, for rotary embeddings. Defaults to `10000`.
        rope_scaling_factor (float, optional): The scaling factor for
            calculation of roatary embedding. Defaults to `1.0`.
        layer_norm_epsilon (float, optional): Epsilon for the layer
            normalization layers in the transformer decoder. Defaults to `1e-6`.
        dtype: string or `keras.mixed_precision.DTypePolicy`. The dtype to use
            for model computations and weights. Note that some computations,
            such as softmax and layer normalization, will always be done at
            float32 precision regardless of dtype.

    Examples:

    ```python
    input_data = {
        "token_ids": np.ones(shape=(1, 12), dtype="int32"),
        "padding_mask": np.array([[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]]),
    }

    # Pretrained Llama decoder.
    model = keras_hub.models.LlamaBackbone.from_preset("llama2_7b_en")
    model(input_data)

    # Randomly initialized Llama decoder with custom config.
    model = keras_hub.models.LlamaBackbone(
        vocabulary_size=10,
        hidden_dim=512,
        num_layers=2,
        num_query_heads=32,
        num_key_value_heads=8,
        intermediate_dim=1024,
        layer_norm_epsilon=1e-6,
        dtype="float32"
    )
    model(input_data)
    ```
    """

    def __init__(
        self,
        vocabulary_size,
        num_layers,
        num_query_heads,
        hidden_dim,
        intermediate_dim,
        num_key_value_heads,
        rope_max_wavelength=10000,
        rope_scaling_factor=1.0,
        layer_norm_epsilon=1e-6,
        dropout=0,
        dtype=None,
        tie_word_embeddings=False,
        **kwargs,
    ):
        # === Layers ===
        self.token_embedding = ReversibleEmbedding(
            input_dim=vocabulary_size,
            output_dim=hidden_dim,
            tie_weights=tie_word_embeddings,
            embeddings_initializer=_llama_kernel_initializer(stddev=0.01),
            dtype=dtype,
            name="token_embedding",
        )
        self.transformer_layers = []
        for i in range(num_layers):
            layer = LlamaTransformerDecoder(
                intermediate_dim=intermediate_dim,
                num_query_heads=num_query_heads,
                num_key_value_heads=num_key_value_heads,
                rope_max_wavelength=rope_max_wavelength,
                rope_scaling_factor=rope_scaling_factor,
                layer_norm_epsilon=layer_norm_epsilon,
                activation=ops.silu,
                kernel_initializer=_llama_kernel_initializer(stddev=0.02),
                dropout=dropout,
                dtype=dtype,
                name=f"transformer_layer_{i}",
            )
            self.transformer_layers.append(layer)
        self.layer_norm = LlamaLayerNorm(
            epsilon=layer_norm_epsilon,
            dtype=dtype,
            name="sequence_output_layernorm",
        )

        # === Functional Model ===
        token_id_input = keras.Input(
            shape=(None,), dtype="int32", name="token_ids"
        )
        padding_mask_input = keras.Input(
            shape=(None,), dtype="int32", name="padding_mask"
        )
        x = self.token_embedding(token_id_input)
        for transformer_layer in self.transformer_layers:
            x = transformer_layer(x, decoder_padding_mask=padding_mask_input)
        sequence_output = self.layer_norm(x)
        super().__init__(
            inputs={
                "token_ids": token_id_input,
                "padding_mask": padding_mask_input,
            },
            outputs=sequence_output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.vocabulary_size = vocabulary_size
        self.num_layers = num_layers
        self.num_query_heads = num_query_heads
        self.hidden_dim = hidden_dim
        self.intermediate_dim = intermediate_dim
        self.rope_max_wavelength = rope_max_wavelength
        self.num_key_value_heads = num_key_value_heads
        self.rope_scaling_factor = rope_scaling_factor
        self.layer_norm_epsilon = layer_norm_epsilon
        self.dropout = dropout
        self.tie_word_embeddings = tie_word_embeddings

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "vocabulary_size": self.vocabulary_size,
                "num_layers": self.num_layers,
                "num_query_heads": self.num_query_heads,
                "hidden_dim": self.hidden_dim,
                "intermediate_dim": self.intermediate_dim,
                "rope_max_wavelength": self.rope_max_wavelength,
                "rope_scaling_factor": self.rope_scaling_factor,
                "num_key_value_heads": self.num_key_value_heads,
                "layer_norm_epsilon": self.layer_norm_epsilon,
                "dropout": self.dropout,
            }
        )
        return config

    @staticmethod
    def get_layout_map(
        device_mesh,
        model_parallel_dim_name="model",
        data_parallel_dim_name="batch",
    ):
        """Get a `keras.distribution.LayoutMap` for model parallel distribution.

        The returned `LayoutMap` contains the sharding spec for the Llama
        backbone weights, so that you can use it to distribute weights across
        the accelerators.

        Example:
        ```
        # Feel free to change the mesh shape to balance data and model
        # parallelism
        mesh = keras.distribution.DeviceMesh(
            shape=(1, 8),
            axis_names=('batch', 'model'),
            devices=keras.distribution.list_devices(),
        )
        layout_map = LlamaBackbone.get_layout_map(
            mesh,
            model_parallel_dim_name="model",
        )

        distribution = keras.distribution.ModelParallel(
            layout_map=layout_map,
            batch_dim_name='batch',
        )

        with distribution.scope():
           llama_model = keras_hub.models.LlamaCausalLM.from_preset()
        ```

        To see how the layout map was applied, load the model then run
        (for one decoder block):
        ```
        embedding_layer = llama_model.backbone.get_layer("token_embedding")
        decoder_block_1 = llama_model.backbone.get_layer('transformer_layer_0')
        for variable in embedding_layer.weights + decoder_block_1.weights:
            print(
                f'{variable.path:<58}  {str(variable.shape):<16}  '
                f'{str(variable.value.sharding.spec)}'
            )
        ```

        Args:
            device_mesh: The `keras.distribution.DeviceMesh` instance for
                distribution.
            model_parallel_dim_name: The axis name of the device mesh, where
                the weights should be partition on.
            data_parallel_dim_name: The axis name of the device mesh, where
                the data should be partition on.
        Return:
            `keras.distribution.LayoutMap` that contains the sharding spec
            for all the model weights.
        """
        # The weight path and shape of the Llama backbone is like below
        # token_embedding/embeddings                              (128256, 2048)
        # repeat block for decoder
        # transformer_layer_0/self_attention/query/kernel         (2048, 32, 64)
        # transformer_layer_0/self_attention/key/kernel           (2048, 8, 64)
        # transformer_layer_0/self_attention/value/kernel         (2048, 8, 64)
        # transformer_layer_0/self_attention/attention_output/kernel
        #                                                         (32, 64, 2048)
        # transformer_layer_0/self_attention_layernorm/scale      (2048,)
        # transformer_layer_0/feedforward_intermediate_dense/kernel
        #                                                         (2048, 8192)
        # transformer_layer_0/feedforward_gate_dense/kernel       (2048, 8192)
        # transformer_layer_0/feedforward_output_dense/kerne      (8192, 2048)
        # transformer_layer_0/feedforward_layernorm/scale         (2048,)

        if not isinstance(device_mesh, keras.distribution.DeviceMesh):
            raise ValueError(
                "Invalid device_mesh type. Expected "
                f"`keras.distribution.Device`, got {type(device_mesh)}"
            )
        if model_parallel_dim_name not in device_mesh.axis_names:
            raise ValueError(
                f"{model_parallel_dim_name} is not found in the "
                f"device_mesh.axis_names. {device_mesh.axis_name=}"
            )
        if data_parallel_dim_name not in device_mesh.axis_names:
            raise ValueError(
                f"{data_parallel_dim_name} is not found in the "
                f"device_mesh.axis_names. {device_mesh.axis_name=}"
            )
        # Note that it is possible to further config the mesh to be 3D, eg
        # (data, seq, model). We leave it as 2D for now for simplicity.
        data_dim = data_parallel_dim_name
        model_dim = model_parallel_dim_name
        # The sharding config is based on the Gemma team training config.
        # See https://arxiv.org/abs/2403.08295
        layout_map = keras.distribution.LayoutMap(device_mesh)
        layout_map["token_embedding/embeddings"] = (model_dim, data_dim)
        layout_map[
            "transformer_layer.*self_attention.*(query|key|value).kernel"
        ] = (
            model_dim,
            data_dim,
            None,
        )
        layout_map["transformer_layer.*attention_output.kernel"] = (
            model_dim,
            None,
            data_dim,
        )
        layout_map[
            "transformer_layer.*feedforward_intermediate_dense.kernel"
        ] = (
            data_dim,
            model_dim,
        )
        layout_map["transformer_layer.*feedforward_gate_dense.kernel"] = (
            data_dim,
            model_dim,
        )
        layout_map["transformer_layer.*feedforward_output_dense.kernel"] = (
            model_dim,
            data_dim,
        )

        return layout_map



import keras
from keras import ops


class T5LayerNorm(keras.layers.Layer):
    def __init__(self, epsilon=1e-6, **kwargs):
        super().__init__(**kwargs)
        self.epsilon = epsilon

    def build(self, input_shape):
        self.weight = self.add_weight(
            name="weight",
            shape=(input_shape[-1],),
            initializer="ones",
        )
        self.built = True

    def call(self, hidden_states):
        variance = ops.mean(ops.square(hidden_states), axis=-1, keepdims=True)
        hidden_states = hidden_states * ops.rsqrt(variance + self.epsilon)
        return self.weight * hidden_states



import keras
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.modeling.anchor_generator import AnchorGenerator
from keras_hub.src.layers.modeling.non_max_supression import NonMaxSuppression
from keras_hub.src.models.object_detector import ObjectDetector
from keras_hub.src.models.retinanet.prediction_head import PredictionHead
from keras_hub.src.models.retinanet.retinanet_backbone import RetinaNetBackbone
from keras_hub.src.models.retinanet.retinanet_label_encoder import (
    RetinaNetLabelEncoder,
)
from keras_hub.src.models.retinanet.retinanet_object_detector_preprocessor import (  # noqa: E501
    RetinaNetObjectDetectorPreprocessor,
)
from keras_hub.src.utils.tensor_utils import assert_bounding_box_support


@keras_hub_export("keras_hub.models.RetinaNetObjectDetector")
class RetinaNetObjectDetector(ObjectDetector):
    """RetinaNet object detector model.

    This class implements the RetinaNet object detection architecture.
    It consists of a feature extractor backbone, a feature pyramid network(FPN),
    and two prediction heads (for classification and bounding box regression).

    Args:
        backbone: `keras.Model`. A `keras.models.RetinaNetBackbone` class,
            defining the backbone network architecture. Provides feature maps
            for detection.
        anchor_generator: A `keras_hub.layers.AnchorGenerator` instance.
            Generates anchor boxes at different scales and aspect ratios
            across the image. If None, a default `AnchorGenerator` is
            created with the following parameters:
                - `bounding_box_format`:  Same as the model's
                   `bounding_box_format`.
                - `min_level`: The backbone's `min_level`.
                - `max_level`: The backbone's `max_level`.
                - `num_scales`: 3.
                - `aspect_ratios`: [0.5, 1.0, 2.0].
                - `anchor_size`: 4.0.
            You can create a custom `AnchorGenerator` by instantiating the
            `keras_hub.layers.AnchorGenerator` class and passing the desired
            arguments.
        num_classes: int. The number of object classes to be detected.
        bounding_box_format: str. Dataset bounding box format (e.g., "xyxy",
            "yxyx"). Defaults to `yxyx`.
        label_encoder: Optional. A `RetinaNetLabelEncoder` instance.  Encodes
            ground truth boxes and classes into training targets. It matches
            ground truth boxes to anchors based on IoU and encodes box
            coordinates as offsets. If `None`, a default encoder is created.
            See the `RetinaNetLabelEncoder` class for details. If None, a
            default encoder is created with standard parameters.
                - `anchor_generator`: Same as the model's.
                - `bounding_box_format`:  Same as the model's
                   `bounding_box_format`.
                - `positive_threshold`: 0.5
                - `negative_threshold`: 0.4
                - `encoding_format`: "center_xywh"
                - `box_variance`: [1.0, 1.0, 1.0, 1.0]
                - `background_class`: -1
                - `ignore_class`: -2
        use_prediction_head_norm: bool. Whether to use Group Normalization after
            the convolution layers in the prediction heads. Defaults to `False`.
        classification_head_prior_probability: float.  Prior probability for the
            classification head (used for focal loss). Defaults to 0.01.
        pre_logits_num_conv_layers: int. The number of convolutional layers in
            the head before the logits layer. These convolutional layers are
            applied before the final linear layer (logits) that produces the
            output predictions (bounding box regressions,
            classification scores).
        preprocessor: Optional. An instance of
            `RetinaNetObjectDetectorPreprocessor`or a custom preprocessor.
            Handles image preprocessing before feeding into the backbone.
        activation: Optional. The activation function to be used in the
            classification head. If None, sigmoid is used.
        dtype: Optional. The data type for the prediction heads. Defaults to the
            backbone's dtype policy.
        prediction_decoder: Optional. A `keras.layers.Layer` instance
            responsible for transforming RetinaNet predictions
            (box regressions and classifications) into final bounding boxes and
            classes with confidence scores. Defaults to a `NonMaxSuppression`
            instance.
    """

    backbone_cls = RetinaNetBackbone
    preprocessor_cls = RetinaNetObjectDetectorPreprocessor

    def __init__(
        self,
        backbone,
        num_classes,
        bounding_box_format="yxyx",
        anchor_generator=None,
        label_encoder=None,
        use_prediction_head_norm=False,
        classification_head_prior_probability=0.01,
        pre_logits_num_conv_layers=4,
        preprocessor=None,
        activation=None,
        dtype=None,
        prediction_decoder=None,
        **kwargs,
    ):
        # Check whether current version of keras support bounding box utils
        assert_bounding_box_support(self.__class__.__name__)

        # === Layers ===
        image_input = keras.layers.Input(backbone.image_shape, name="images")
        head_dtype = dtype or backbone.dtype_policy

        anchor_generator = anchor_generator or AnchorGenerator(
            bounding_box_format,
            min_level=backbone.min_level,
            max_level=backbone.max_level,
            num_scales=3,
            aspect_ratios=[0.5, 1.0, 2.0],
            anchor_size=4,
        )
        # As weights are ported from torch they use encoded format
        # as "center_xywh"
        label_encoder = label_encoder or RetinaNetLabelEncoder(
            anchor_generator,
            bounding_box_format=bounding_box_format,
            encoding_format="center_xywh",
        )

        box_head = PredictionHead(
            output_filters=anchor_generator.num_base_anchors * 4,
            num_conv_layers=pre_logits_num_conv_layers,
            num_filters=256,
            use_group_norm=use_prediction_head_norm,
            use_prior_probability=True,
            prior_probability=classification_head_prior_probability,
            dtype=head_dtype,
            name="box_head",
        )
        classification_head = PredictionHead(
            output_filters=anchor_generator.num_base_anchors * num_classes,
            num_conv_layers=pre_logits_num_conv_layers,
            num_filters=256,
            use_group_norm=use_prediction_head_norm,
            dtype=head_dtype,
            name="classification_head",
        )

        # === Functional Model ===
        feature_map = backbone(image_input)

        class_predictions = []
        box_predictions = []

        # Iterate through the feature pyramid levels (e.g., P3, P4, P5, P6, P7).
        for level in feature_map:
            box_predictions.append(
                keras.layers.Reshape((-1, 4), name=f"box_pred_{level}")(
                    box_head(feature_map[level])
                )
            )
            class_predictions.append(
                keras.layers.Reshape(
                    (-1, num_classes), name=f"cls_pred_{level}"
                )(classification_head(feature_map[level]))
            )

        # Concatenate predictions from all FPN levels.
        class_predictions = keras.layers.Concatenate(axis=1, name="cls_logits")(
            class_predictions
        )
        # box_pred is always in "center_xywh" delta-encoded no matter what
        # format you pass in.
        box_predictions = keras.layers.Concatenate(
            axis=1, name="bbox_regression"
        )(box_predictions)

        outputs = {
            "bbox_regression": box_predictions,
            "cls_logits": class_predictions,
        }

        super().__init__(
            inputs=image_input,
            outputs=outputs,
            **kwargs,
        )

        # === Config ===
        self.bounding_box_format = bounding_box_format
        self.use_prediction_head_norm = use_prediction_head_norm
        self.num_classes = num_classes
        self.backbone = backbone
        self.preprocessor = preprocessor
        self.activation = activation
        self.pre_logits_num_conv_layers = pre_logits_num_conv_layers
        self.box_head = box_head
        self.classification_head = classification_head
        self.anchor_generator = anchor_generator
        self.label_encoder = label_encoder
        self._prediction_decoder = prediction_decoder or NonMaxSuppression(
            from_logits=(activation != keras.activations.sigmoid),
            bounding_box_format=bounding_box_format,
        )

    def compute_loss(self, x, y, y_pred, sample_weight, **kwargs):
        _, height, width, _ = keras.ops.shape(x)
        y_for_label_encoder = keras.utils.bounding_boxes.convert_format(
            y,
            source=self.bounding_box_format,
            target=self.label_encoder.bounding_box_format,
            height=height,
            width=width,
        )

        boxes, labels = self.label_encoder(
            images=x,
            gt_boxes=y_for_label_encoder["boxes"],
            gt_classes=y_for_label_encoder["labels"],
        )

        box_pred = y_pred["bbox_regression"]
        cls_pred = y_pred["cls_logits"]

        if boxes.shape[-1] != 4:
            raise ValueError(
                "boxes should have shape (None, None, 4). Got "
                f"boxes.shape={tuple(boxes.shape)}"
            )

        if box_pred.shape[-1] != 4:
            raise ValueError(
                "box_pred should have shape (None, None, 4). Got "
                f"box_pred.shape={tuple(box_pred.shape)}. Does your model's "
                "`num_classes` parameter match your losses `num_classes` "
                "parameter?"
            )
        if cls_pred.shape[-1] != self.num_classes:
            raise ValueError(
                "cls_pred should have shape (None, None, 4). Got "
                f"cls_pred.shape={tuple(cls_pred.shape)}. Does your model's "
                "`num_classes` parameter match your losses `num_classes` "
                "parameter?"
            )

        cls_labels = ops.one_hot(
            ops.cast(labels, "int32"), self.num_classes, dtype="float32"
        )
        positive_mask = ops.cast(ops.greater(labels, -1.0), dtype="float32")
        normalizer = ops.sum(positive_mask)
        cls_weights = ops.cast(ops.not_equal(labels, -2.0), dtype="float32")
        cls_weights /= normalizer
        box_weights = positive_mask / normalizer

        y_true = {
            "bbox_regression": boxes,
            "cls_logits": cls_labels,
        }
        sample_weights = {
            "bbox_regression": box_weights,
            "cls_logits": cls_weights,
        }
        zero_weight = {
            "bbox_regression": ops.zeros_like(box_weights),
            "cls_logits": ops.zeros_like(cls_weights),
        }

        sample_weight = ops.cond(
            normalizer == 0,
            lambda: zero_weight,
            lambda: sample_weights,
        )
        return super().compute_loss(
            x=x, y=y_true, y_pred=y_pred, sample_weight=sample_weight, **kwargs
        )

    def predict_step(self, *args):
        outputs = super().predict_step(*args)
        if isinstance(outputs, tuple):
            return self.decode_predictions(outputs[0], args[-1]), outputs[1]
        return self.decode_predictions(outputs, *args)

    @property
    def prediction_decoder(self):
        return self._prediction_decoder

    @prediction_decoder.setter
    def prediction_decoder(self, prediction_decoder):
        if prediction_decoder.bounding_box_format != self.bounding_box_format:
            raise ValueError(
                "Expected `prediction_decoder` and `RetinaNet` to "
                "use the same `bounding_box_format`, but got "
                "`prediction_decoder.bounding_box_format="
                f"{prediction_decoder.bounding_box_format}`, and "
                "`self.bounding_box_format="
                f"{self.bounding_box_format}`."
            )
        self._prediction_decoder = prediction_decoder
        self.make_predict_function(force=True)
        self.make_train_function(force=True)
        self.make_test_function(force=True)

    def decode_predictions(self, predictions, data):
        box_pred = predictions["bbox_regression"]
        cls_pred = predictions["cls_logits"]
        # box_pred is on "center_yxhw" format, convert to target format.
        if isinstance(data, list) or isinstance(data, tuple):
            images, _ = data
        else:
            images = data
        height, width, channels = ops.shape(images)[1:]
        anchor_boxes = self.anchor_generator(images)
        anchor_boxes = ops.concatenate(list(anchor_boxes.values()), axis=0)
        box_pred = keras.utils.bounding_boxes.decode_deltas_to_boxes(
            anchors=anchor_boxes,
            boxes_delta=box_pred,
            encoded_format="center_xywh",
            anchor_format=self.anchor_generator.bounding_box_format,
            box_format=self.bounding_box_format,
            image_shape=(height, width, channels),
        )
        # box_pred is now in "self.bounding_box_format" format
        box_pred = keras.utils.bounding_boxes.convert_format(
            box_pred,
            source=self.bounding_box_format,
            target=self.prediction_decoder.bounding_box_format,
            height=height,
            width=width,
        )
        y_pred = self.prediction_decoder(box_pred, cls_pred, images=images)
        y_pred["boxes"] = keras.utils.bounding_boxes.convert_format(
            y_pred["boxes"],
            source=self.prediction_decoder.bounding_box_format,
            target=self.bounding_box_format,
            height=height,
            width=width,
        )
        return y_pred

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_classes": self.num_classes,
                "use_prediction_head_norm": self.use_prediction_head_norm,
                "pre_logits_num_conv_layers": self.pre_logits_num_conv_layers,
                "bounding_box_format": self.bounding_box_format,
                "anchor_generator": keras.layers.serialize(
                    self.anchor_generator
                ),
                "label_encoder": keras.layers.serialize(self.label_encoder),
                "prediction_decoder": keras.layers.serialize(
                    self._prediction_decoder
                ),
            }
        )

        return config

    @classmethod
    def from_config(cls, config):
        if "label_encoder" in config and isinstance(
            config["label_encoder"], dict
        ):
            config["label_encoder"] = keras.layers.deserialize(
                config["label_encoder"]
            )

        if "anchor_generator" in config and isinstance(
            config["anchor_generator"], dict
        ):
            config["anchor_generator"] = keras.layers.deserialize(
                config["anchor_generator"]
            )

        if "prediction_decoder" in config and isinstance(
            config["prediction_decoder"], dict
        ):
            config["prediction_decoder"] = keras.layers.deserialize(
                config["prediction_decoder"]
            )

        return super().from_config(config)



"""EfficientNet preset configurations."""

backbone_presets = {
    "efficientnet_b0_ra_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B0 model pre-trained on the ImageNet 1k dataset "
                "with RandAugment recipe."
            ),
            "params": 5288548,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b0_ra_imagenet/2",
    },
    "efficientnet_b0_ra4_e3600_r224_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B0 model pre-trained on the ImageNet 1k dataset "
                "by Ross Wightman. Trained with timm scripts using "
                "hyper-parameters inspired by the MobileNet-V4 small, mixed "
                "with go-to hparams from timm and 'ResNet Strikes Back'."
            ),
            "params": 5288548,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b0_ra4_e3600_r224_imagenet/2",
    },
    "efficientnet_b1_ft_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B1 model fine-tuned on the ImageNet 1k dataset."
            ),
            "params": 7794184,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b1_ft_imagenet/5",
    },
    "efficientnet_b1_ra4_e3600_r240_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B1 model pre-trained on the ImageNet 1k dataset "
                "by Ross Wightman. Trained with timm scripts using "
                "hyper-parameters inspired by the MobileNet-V4 small, mixed "
                "with go-to hparams from timm and 'ResNet Strikes Back'."
            ),
            "params": 7794184,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b1_ra4_e3600_r240_imagenet/2",
    },
    "efficientnet_b2_ra_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B2 model pre-trained on the ImageNet 1k dataset "
                "with RandAugment recipe."
            ),
            "params": 9109994,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b2_ra_imagenet/2",
    },
    "efficientnet_b3_ra2_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B3 model pre-trained on the ImageNet 1k dataset "
                "with RandAugment2 recipe."
            ),
            "params": 12233232,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b3_ra2_imagenet/2",
    },
    "efficientnet_b4_ra2_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B4 model pre-trained on the ImageNet 1k dataset "
                "with RandAugment2 recipe."
            ),
            "params": 19341616,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b4_ra2_imagenet/2",
    },
    "efficientnet_b5_sw_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B5 model pre-trained on the ImageNet 12k dataset "
                "by Ross Wightman. Based on Swin Transformer train / pretrain "
                "recipe with modifications (related to both DeiT and ConvNeXt "
                "recipes)."
            ),
            "params": 30389784,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b5_sw_imagenet/2",
    },
    "efficientnet_b5_sw_ft_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet B5 model pre-trained on the ImageNet 12k dataset "
                "and fine-tuned on ImageNet-1k by Ross Wightman. Based on Swin "
                "Transformer train / pretrain recipe with modifications "
                "(related to both DeiT and ConvNeXt recipes)."
            ),
            "params": 30389784,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b5_sw_ft_imagenet/2",
    },
    "efficientnet_el_ra_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-EdgeTPU Large model trained on the ImageNet 1k "
                "dataset with RandAugment recipe."
            ),
            "params": 10589712,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b1_ft_imagenet/5",
    },
    "efficientnet_em_ra2_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-EdgeTPU Medium model trained on the ImageNet 1k "
                "dataset with RandAugment2 recipe."
            ),
            "params": 6899496,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b1_ft_imagenet/5",
    },
    "efficientnet_es_ra_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-EdgeTPU Small model trained on the ImageNet 1k "
                "dataset with RandAugment recipe."
            ),
            "params": 5438392,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_b1_ft_imagenet/5",
    },
    "efficientnet2_rw_m_agc_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-v2 Medium model trained on the ImageNet 1k "
                "dataset with adaptive gradient clipping."
            ),
            "params": 53236442,
            "official_name": "EfficientNet",
            "path": "efficientnet",
            "model_card": "https://arxiv.org/abs/2104.00298",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet2_rw_m_agc_imagenet/2",
    },
    "efficientnet2_rw_s_ra2_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-v2 Small model trained on the ImageNet 1k "
                "dataset with RandAugment2 recipe."
            ),
            "params": 23941296,
            "official_name": "EfficientNet",
            "path": "efficientnet",
            "model_card": "https://arxiv.org/abs/2104.00298",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet2_rw_s_ra2_imagenet/2",
    },
    "efficientnet2_rw_t_ra2_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-v2 Tiny model trained on the ImageNet 1k "
                "dataset with RandAugment2 recipe."
            ),
            "params": 13649388,
            "official_name": "EfficientNet",
            "path": "efficientnet",
            "model_card": "https://arxiv.org/abs/2104.00298",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet2_rw_t_ra2_imagenet/2",
    },
    "efficientnet_lite0_ra_imagenet": {
        "metadata": {
            "description": (
                "EfficientNet-Lite model fine-trained on the ImageNet 1k "
                "dataset with RandAugment recipe."
            ),
            "params": 4652008,
            "path": "efficientnet",
        },
        "kaggle_handle": "kaggle://keras/efficientnet/keras/efficientnet_lite0_ra_imagenet/2",
    },
}



"""BASNet model preset configurations."""

basnet_presets = {
    "basnet_duts": {
        "metadata": {
            "description": (
                "BASNet model with a 34-layer ResNet backbone, pre-trained "
                "on the DUTS image dataset at a 288x288 resolution. Model "
                "training was performed by Hamid Ali "
                "(https://github.com/hamidriasat/BASNet)."
            ),
            "params": 108886792,
            "path": "basnet",
        },
        "kaggle_handle": "kaggle://keras/basnet/keras/basnet_duts",
    },
}



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.models.efficientnet.efficientnet_backbone import (
    EfficientNetBackbone,
)
from keras_hub.src.models.efficientnet.efficientnet_image_classifier_preprocessor import (  # noqa: E501
    EfficientNetImageClassifierPreprocessor,
)
from keras_hub.src.models.image_classifier import ImageClassifier


@keras_hub_export("keras_hub.models.EfficientNetImageClassifier")
class EfficientNetImageClassifier(ImageClassifier):
    backbone_cls = EfficientNetBackbone
    preprocessor_cls = EfficientNetImageClassifierPreprocessor



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.mit import MiTBackbone


@keras_hub_export("keras_hub.layers.MiTImageConverter")
class MiTImageConverter(ImageConverter):
    backbone_cls = MiTBackbone



"""BART model preset configurations."""

backbone_presets = {
    "bart_base_en": {
        "metadata": {
            "description": (
                "6-layer BART model where case is maintained. "
                "Trained on BookCorpus, English Wikipedia and CommonCrawl."
            ),
            "params": 139417344,
            "path": "bart",
        },
        "kaggle_handle": "kaggle://keras/bart/keras/bart_base_en/3",
    },
    "bart_large_en": {
        "metadata": {
            "description": (
                "12-layer BART model where case is maintained. "
                "Trained on BookCorpus, English Wikipedia and CommonCrawl."
            ),
            "params": 406287360,
            "path": "bart",
        },
        "config": {
            "vocabulary_size": 50265,
            "num_layers": 12,
            "num_heads": 16,
            "hidden_dim": 1024,
            "intermediate_dim": 4096,
            "dropout": 0.1,
            "max_sequence_length": 1024,
        },
        "kaggle_handle": "kaggle://keras/bart/keras/bart_large_en/3",
    },
    "bart_large_en_cnn": {
        "metadata": {
            "description": (
                "The `bart_large_en` backbone model fine-tuned on the CNN+DM "
                "summarization dataset."
            ),
            "params": 406287360,
            "path": "bart",
        },
        "config": {
            "vocabulary_size": 50264,
            "num_layers": 12,
            "num_heads": 16,
            "hidden_dim": 1024,
            "intermediate_dim": 4096,
            "dropout": 0.1,
            "max_sequence_length": 1024,
        },
        "kaggle_handle": "kaggle://keras/bart/keras/bart_large_en_cnn/3",
    },
}



from keras_hub.src.models.segformer.segformer_backbone import SegFormerBackbone
from keras_hub.src.models.segformer.segformer_image_segmenter import (
    SegFormerImageSegmenter,
)
from keras_hub.src.models.segformer.segformer_presets import presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(presets, SegFormerImageSegmenter)



"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.utils.imagenet.imagenet_utils import (
    decode_imagenet_predictions,
)



import keras

from keras_hub.src.models.backbone import Backbone
from keras_hub.src.models.vae.vae_layers import (
    DiagonalGaussianDistributionSampler,
)
from keras_hub.src.models.vae.vae_layers import VAEDecoder
from keras_hub.src.models.vae.vae_layers import VAEEncoder
from keras_hub.src.utils.keras_utils import standardize_data_format


class VAEBackbone(Backbone):
    """Variational Autoencoder(VAE) backbone used in latent diffusion models.

    When encoding, this model generates mean and log variance of the input
    images. When decoding, it reconstructs images from the latent space.

    Args:
        encoder_num_filters: list of ints. The number of filters for each
            block in encoder.
        encoder_num_blocks: list of ints. The number of blocks for each block in
            encoder.
        decoder_num_filters: list of ints. The number of filters for each
            block in decoder.
        decoder_num_blocks: list of ints. The number of blocks for each block in
            decoder.
        sampler_method: str. The method of the sampler for the intermediate
            output. Available methods are `"sample"` and `"mode"`. `"sample"`
            draws from the distribution using both the mean and log variance.
            `"mode"` draws from the distribution using the mean only. Defaults
            to `sample`.
        input_channels: int. The number of channels in the input.
        sample_channels: int. The number of channels in the sample. Typically,
            this indicates the intermediate output of VAE, which is mean and
            log variance.
        output_channels: int. The number of channels in the output.
        scale: float. The scaling factor applied to the latent space to ensure
            it has unit variance during training of the diffusion model.
            Defaults to `1.5305`, which is the value used in Stable Diffusion 3.
        shift: float. The shift factor applied to the latent space to ensure it
            has zero mean during training of the diffusion model. Defaults to
            `0.0609`, which is the value used in Stable Diffusion 3.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        dtype: `None` or str or `keras.mixed_precision.DTypePolicy`. The dtype
            to use for the model's computations and weights.

    Example:
    ```Python
    backbone = VAEBackbone(
        encoder_num_filters=[32, 32, 32, 32],
        encoder_num_blocks=[1, 1, 1, 1],
        decoder_num_filters=[32, 32, 32, 32],
        decoder_num_blocks=[1, 1, 1, 1],
    )
    input_data = ops.ones((2, self.height, self.width, 3))
    output = backbone(input_data)
    ```
    """

    def __init__(
        self,
        encoder_num_filters,
        encoder_num_blocks,
        decoder_num_filters,
        decoder_num_blocks,
        sampler_method="sample",
        input_channels=3,
        sample_channels=32,
        output_channels=3,
        scale=1.5305,
        shift=0.0609,
        data_format=None,
        dtype=None,
        **kwargs,
    ):
        data_format = standardize_data_format(data_format)
        if data_format == "channels_last":
            image_shape = (None, None, input_channels)
            channel_axis = -1
        else:
            image_shape = (input_channels, None, None)
            channel_axis = 1

        # === Layers ===
        self.encoder = VAEEncoder(
            encoder_num_filters,
            encoder_num_blocks,
            output_channels=sample_channels,
            data_format=data_format,
            dtype=dtype,
            name="encoder",
        )
        # Use `sample()` to define the functional model.
        self.distribution_sampler = DiagonalGaussianDistributionSampler(
            method=sampler_method,
            axis=channel_axis,
            dtype=dtype,
            name="distribution_sampler",
        )
        self.decoder = VAEDecoder(
            decoder_num_filters,
            decoder_num_blocks,
            output_channels=output_channels,
            data_format=data_format,
            dtype=dtype,
            name="decoder",
        )

        # === Functional Model ===
        image_input = keras.Input(shape=image_shape)
        sample = self.encoder(image_input)
        latent = self.distribution_sampler(sample)
        image_output = self.decoder(latent)
        super().__init__(
            inputs=image_input,
            outputs=image_output,
            dtype=dtype,
            **kwargs,
        )

        # === Config ===
        self.encoder_num_filters = encoder_num_filters
        self.encoder_num_blocks = encoder_num_blocks
        self.decoder_num_filters = decoder_num_filters
        self.decoder_num_blocks = decoder_num_blocks
        self.sampler_method = sampler_method
        self.input_channels = input_channels
        self.sample_channels = sample_channels
        self.output_channels = output_channels
        self._scale = scale
        self._shift = shift

    @property
    def scale(self):
        """The scaling factor for the latent space.

        This is used to scale the latent space to have unit variance when
        training the diffusion model.
        """
        return self._scale

    @property
    def shift(self):
        """The shift factor for the latent space.

        This is used to shift the latent space to have zero mean when
        training the diffusion model.
        """
        return self._shift

    def encode(self, inputs, **kwargs):
        """Encode the input images into latent space."""
        sample = self.encoder(inputs, **kwargs)
        return self.distribution_sampler(sample)

    def decode(self, inputs, **kwargs):
        """Decode the input latent space into images."""
        return self.decoder(inputs, **kwargs)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "encoder_num_filters": self.encoder_num_filters,
                "encoder_num_blocks": self.encoder_num_blocks,
                "decoder_num_filters": self.decoder_num_filters,
                "decoder_num_blocks": self.decoder_num_blocks,
                "sampler_method": self.sampler_method,
                "input_channels": self.input_channels,
                "sample_channels": self.sample_channels,
                "output_channels": self.output_channels,
                "scale": self.scale,
                "shift": self.shift,
            }
        )
        return config



from keras_hub.src.models.roberta.roberta_backbone import RobertaBackbone
from keras_hub.src.models.roberta.roberta_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, RobertaBackbone)



import keras
from keras import ops

from keras_hub.src.utils.keras_utils import standardize_data_format


class MLP(keras.layers.Layer):
    """Multi-Layer Perceptron (MLP) block.

    Args:
        hidden_dim: int. Dimensionality of the hidden representations.
        mlp_dim: int. Dimensionality of the intermediate MLP layer.
        use_bias: bool. Whether to use bias in the dense layers. Defaults to
            `True`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to `0.0`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        hidden_dim,
        mlp_dim,
        use_bias=True,
        dropout_rate=0.0,
        **kwargs,
    ):
        super().__init__(**kwargs)

        # === Config ===
        self.hidden_dim = hidden_dim
        self.mlp_dim = mlp_dim
        self.use_bias = use_bias
        self.dropout_rate = dropout_rate

    def build(self, input_shape):
        self.dense_1 = keras.layers.Dense(
            units=self.mlp_dim,
            use_bias=self.use_bias,
            activation="gelu",
            bias_initializer=(
                keras.initializers.RandomNormal(stddev=1e-6)
                if self.use_bias
                else None
            ),
            dtype=self.dtype_policy,
            name="dense_1",
        )
        self.dense_1.build(input_shape)
        self.dense_2 = keras.layers.Dense(
            units=self.hidden_dim,
            use_bias=self.use_bias,
            bias_initializer=(
                keras.initializers.RandomNormal(stddev=1e-6)
                if self.use_bias
                else None
            ),
            dtype=self.dtype_policy,
            name="dense_2",
        )
        self.dense_2.build((None, None, self.mlp_dim))
        self.dropout = keras.layers.Dropout(
            self.dropout_rate, dtype=self.dtype_policy, name="dropout"
        )
        self.built = True

    def call(self, inputs):
        x = self.dense_1(inputs)
        x = self.dropout(x)
        x = self.dense_2(x)
        out = self.dropout(x)
        return out


class ViTPatchingAndEmbedding(keras.layers.Layer):
    """Patches the image and embeds the patches.

    Args:
        image_size: int. Size of the input image (height or width).
            Assumed to be square.
        patch_size: int. Size of each image patch.
        hidden_dim: int. Dimensionality of the patch embeddings.
        num_channels: int. Number of channels in the input image. Defaults to
            `3`.
        data_format: str. `"channels_last"` or `"channels_first"`. Defaults to
            `None` (which uses `"channels_last"`).
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        image_size,
        patch_size,
        hidden_dim,
        num_channels=3,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        num_patches = (image_size // patch_size) ** 2
        num_positions = num_patches + 1

        # === Config ===
        self.image_size = image_size
        self.patch_size = patch_size
        self.hidden_dim = hidden_dim
        self.num_channels = num_channels
        self.num_patches = num_patches
        self.num_positions = num_positions
        self.data_format = standardize_data_format(data_format)

    def build(self, input_shape):
        self.class_token = self.add_weight(
            shape=(
                1,
                1,
                self.hidden_dim,
            ),
            initializer="random_normal",
            dtype=self.variable_dtype,
            name="class_token",
        )
        self.patch_embedding = keras.layers.Conv2D(
            filters=self.hidden_dim,
            kernel_size=self.patch_size,
            strides=self.patch_size,
            padding="valid",
            activation=None,
            dtype=self.dtype_policy,
            data_format=self.data_format,
            name="patch_embedding",
        )
        self.patch_embedding.build(input_shape)
        self.position_embedding = keras.layers.Embedding(
            self.num_positions,
            self.hidden_dim,
            dtype=self.dtype_policy,
            embeddings_initializer=keras.initializers.RandomNormal(stddev=0.02),
            name="position_embedding",
        )
        self.position_embedding.build((1, self.num_positions))
        self.position_ids = keras.ops.expand_dims(
            keras.ops.arange(self.num_positions), axis=0
        )
        self.built = True

    def call(self, inputs):
        patch_embeddings = self.patch_embedding(inputs)
        if self.data_format == "channels_first":
            patch_embeddings = ops.transpose(
                patch_embeddings, axes=(0, 2, 3, 1)
            )
        embeddings_shape = ops.shape(patch_embeddings)
        patch_embeddings = ops.reshape(
            patch_embeddings, [embeddings_shape[0], -1, embeddings_shape[-1]]
        )
        class_token = ops.tile(self.class_token, (embeddings_shape[0], 1, 1))
        position_embeddings = self.position_embedding(self.position_ids)
        embeddings = ops.concatenate([class_token, patch_embeddings], axis=1)
        return ops.add(embeddings, position_embeddings)

    def compute_output_shape(self, input_shape):
        return (
            input_shape[0],
            self.num_positions,
            self.hidden_dim,
        )

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_size": self.image_size,
                "patch_size": self.patch_size,
                "hidden_dim": self.hidden_dim,
                "num_channels": self.num_channels,
                "num_patches": self.num_patches,
                "num_positions": self.num_positions,
            }
        )
        return config


class ViTEncoderBlock(keras.layers.Layer):
    """Transformer encoder block.

    Args:
        num_heads: int. Number of attention heads.
        hidden_dim: int. Dimensionality of the hidden representations.
        mlp_dim: int. Dimensionality of the intermediate MLP layer.
        use_mha_bias: bool. Whether to use bias in the multi-head attention
            layer. Defaults to `True`.
        use_mlp_bias: bool. Whether to use bias in the MLP layer. Defaults to
            `True`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to `0.0`.
        attention_dropout: float. Dropout rate for the attention mechanism.
            Between 0 and 1. Defaults to `0.0`.
        layer_norm_epsilon: float. Small float value for layer normalization
            stability. Defaults to `1e-6`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        num_heads,
        hidden_dim,
        mlp_dim,
        use_mha_bias=True,
        use_mlp_bias=True,
        dropout_rate=0.0,
        attention_dropout=0.0,
        layer_norm_epsilon=1e-6,
        **kwargs,
    ):
        super().__init__(**kwargs)

        key_dim = hidden_dim // num_heads

        # === Config ===
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.key_dim = key_dim
        self.mlp_dim = mlp_dim
        self.use_mha_bias = use_mha_bias
        self.use_mlp_bias = use_mlp_bias
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout
        self.layer_norm_epsilon = layer_norm_epsilon

    def build(self, input_shape):
        # Attention block
        self.layer_norm_1 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            name="ln_1",
            dtype=self.dtype_policy,
        )
        self.layer_norm_1.build(input_shape)
        self.mha = keras.layers.MultiHeadAttention(
            num_heads=self.num_heads,
            key_dim=self.key_dim,
            use_bias=self.use_mha_bias,
            dropout=self.attention_dropout,
            name="mha",
            dtype=self.dtype_policy,
        )
        self.mha.build(input_shape, input_shape)
        self.dropout = keras.layers.Dropout(
            self.dropout_rate, dtype=self.dtype_policy, name="dropout"
        )

        # MLP block
        self.layer_norm_2 = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            name="ln_2",
            dtype=self.dtype_policy,
        )
        self.layer_norm_2.build((None, None, self.hidden_dim))
        self.mlp = MLP(
            hidden_dim=self.hidden_dim,
            mlp_dim=self.mlp_dim,
            use_bias=self.use_mlp_bias,
            dropout_rate=self.dropout_rate,
            name="mlp",
            dtype=self.dtype_policy,
        )
        self.mlp.build((None, None, self.hidden_dim))
        self.built = True

    def call(self, inputs):
        x = self.layer_norm_1(inputs)
        x = self.mha(x, x)
        x = self.dropout(x)
        x = x + inputs

        y = self.layer_norm_2(x)
        y = self.mlp(y)

        return x + y

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "key_dim": self.key_dim,
                "mlp_dim": self.mlp_dim,
                "use_mha_bias": self.use_mha_bias,
                "use_mlp_bias": self.use_mlp_bias,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config


class ViTEncoder(keras.layers.Layer):
    """Vision Transformer (ViT) encoder.

    Args:
        num_layers: int. Number of Transformer encoder blocks.
        num_heads: int. Number of attention heads.
        hidden_dim: int. Dimensionality of the hidden representations.
        mlp_dim: int. Dimensionality of the intermediate MLP layer.
        use_mha_bias: bool. Whether to use bias in the multi-head attention
            layers. Defaults to `True`.
        use_mlp_bias: bool. Whether to use bias in the MLP layers. Defaults to
            `True`.
        dropout_rate: float. Dropout rate. Between 0 and 1. Defaults to `0.0`.
        attention_dropout: float. Dropout rate for the attention mechanism.
            Between 0 and 1. Defaults to `0.0`.
        layer_norm_epsilon: float. Small float value for layer normalization
            tability. Defaults to `1e-6`.
        **kwargs: Additional keyword arguments passed to `keras.layers.Layer`
    """

    def __init__(
        self,
        num_layers,
        num_heads,
        hidden_dim,
        mlp_dim,
        use_mha_bias=True,
        use_mlp_bias=True,
        dropout_rate=0.0,
        attention_dropout=0.0,
        layer_norm_epsilon=1e-6,
        **kwargs,
    ):
        super().__init__(**kwargs)

        # === config ===
        self.num_layers = num_layers
        self.num_heads = num_heads
        self.hidden_dim = hidden_dim
        self.mlp_dim = mlp_dim
        self.use_mha_bias = use_mha_bias
        self.use_mlp_bias = use_mlp_bias
        self.dropout_rate = dropout_rate
        self.attention_dropout = attention_dropout
        self.layer_norm_epsilon = layer_norm_epsilon

    def build(self, input_shape):
        self.encoder_layers = []
        for i in range(self.num_layers):
            encoder_block = ViTEncoderBlock(
                num_heads=self.num_heads,
                hidden_dim=self.hidden_dim,
                mlp_dim=self.mlp_dim,
                dropout_rate=self.dropout_rate,
                use_mha_bias=self.use_mha_bias,
                use_mlp_bias=self.use_mlp_bias,
                attention_dropout=self.attention_dropout,
                layer_norm_epsilon=self.layer_norm_epsilon,
                dtype=self.dtype_policy,
                name=f"tranformer_block_{i + 1}",
            )
            encoder_block.build((None, None, self.hidden_dim))
            self.encoder_layers.append(encoder_block)
        self.dropout = keras.layers.Dropout(
            self.dropout_rate, dtype=self.dtype_policy, name="dropout"
        )
        self.layer_norm = keras.layers.LayerNormalization(
            epsilon=self.layer_norm_epsilon,
            dtype=self.dtype_policy,
            name="ln",
        )
        self.layer_norm.build((None, None, self.hidden_dim))
        self.built = True

    def call(self, inputs):
        x = self.dropout(inputs)
        for i in range(self.num_layers):
            x = self.encoder_layers[i](x)
        x = self.layer_norm(x)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "num_layers": self.num_layers,
                "num_heads": self.num_heads,
                "hidden_dim": self.hidden_dim,
                "mlp_dim": self.mlp_dim,
                "use_mha_bias": self.use_mha_bias,
                "use_mlp_bias": self.use_mlp_bias,
                "dropout_rate": self.dropout_rate,
                "attention_dropout": self.attention_dropout,
                "layer_norm_epsilon": self.layer_norm_epsilon,
            }
        )
        return config



import json
import os
import pathlib
import re

import keras
import numpy as np
import tensorflow as tf
from absl.testing import parameterized
from keras import ops
from keras import tree

from keras_hub.src.layers.modeling.reversible_embedding import (
    ReversibleEmbedding,
)
from keras_hub.src.models.retinanet.feature_pyramid import FeaturePyramid
from keras_hub.src.tokenizers.tokenizer import Tokenizer
from keras_hub.src.utils.tensor_utils import is_float_dtype


def convert_to_comparible_type(x):
    """Convert tensors to comparable types.

    Any string are converted to plain python types. Any jax or torch tensors
    are converted to numpy.
    """
    if getattr(x, "dtype", None) == tf.string:
        if isinstance(x, tf.RaggedTensor):
            x = x.to_list()
        if isinstance(x, tf.Tensor):
            x = x.numpy() if x.shape.rank == 0 else x.numpy().tolist()
        return tree.map_structure(lambda x: x.decode("utf-8"), x)
    if isinstance(x, (tf.Tensor, tf.RaggedTensor)):
        return x
    if hasattr(x, "__array__"):
        return ops.convert_to_numpy(x)
    return x


class TestCase(tf.test.TestCase, parameterized.TestCase):
    """Base test case class for KerasHub."""

    def assertAllClose(self, x1, x2, atol=1e-6, rtol=1e-6, msg=None):
        # This metric dict hack is only needed for tf.keras, and can be
        # removed after we fully migrate to keras-core/Keras 3.
        if x1.__class__.__name__ == "_MetricDict":
            x1 = dict(x1)
        if x2.__class__.__name__ == "_MetricDict":
            x2 = dict(x2)
        x1 = tree.map_structure(convert_to_comparible_type, x1)
        x2 = tree.map_structure(convert_to_comparible_type, x2)
        super().assertAllClose(x1, x2, atol=atol, rtol=rtol, msg=msg)

    def assertEqual(self, x1, x2, msg=None):
        x1 = tree.map_structure(convert_to_comparible_type, x1)
        x2 = tree.map_structure(convert_to_comparible_type, x2)
        super().assertEqual(x1, x2, msg=msg)

    def assertAllEqual(self, x1, x2, msg=None):
        x1 = tree.map_structure(convert_to_comparible_type, x1)
        x2 = tree.map_structure(convert_to_comparible_type, x2)
        super().assertAllEqual(x1, x2, msg=msg)

    def assertDTypeEqual(self, x, expected_dtype, msg=None):
        input_dtype = keras.backend.standardize_dtype(x.dtype)
        super().assertEqual(input_dtype, expected_dtype, msg=msg)

    def run_layer_test(
        self,
        cls,
        init_kwargs,
        input_data,
        expected_output_shape,
        expected_output_data=None,
        expected_num_trainable_weights=0,
        expected_num_non_trainable_weights=0,
        expected_num_non_trainable_variables=0,
        run_training_check=True,
        run_precision_checks=True,
    ):
        """Run basic tests for a modeling layer."""
        # Serialization test.
        layer = cls(**init_kwargs)
        self.run_serialization_test(layer)

        def run_build_asserts(layer):
            self.assertTrue(layer.built)
            self.assertLen(
                layer.trainable_weights,
                expected_num_trainable_weights,
                msg="Unexpected number of trainable_weights",
            )
            self.assertLen(
                layer.non_trainable_weights,
                expected_num_non_trainable_weights,
                msg="Unexpected number of non_trainable_weights",
            )
            self.assertLen(
                layer.non_trainable_variables,
                expected_num_non_trainable_variables,
                msg="Unexpected number of non_trainable_variables",
            )

        def run_output_asserts(layer, output, eager=False):
            output_shape = tree.map_structure(
                lambda x: None if x is None else x.shape, output
            )
            self.assertEqual(
                expected_output_shape,
                output_shape,
                msg="Unexpected output shape",
            )
            output_dtype = tree.flatten(output)[0].dtype
            self.assertEqual(
                keras.backend.standardize_dtype(layer.dtype),
                keras.backend.standardize_dtype(output_dtype),
                msg="Unexpected output dtype",
            )
            if eager and expected_output_data is not None:
                self.assertAllClose(expected_output_data, output)

        def run_training_step(layer, input_data, output_data):
            class TestModel(keras.Model):
                def __init__(self, layer):
                    super().__init__()
                    self.layer = layer

                def call(self, x):
                    if isinstance(x, dict):
                        if isinstance(layer, FeaturePyramid):
                            return self.layer(x)
                        else:
                            return self.layer(**x)
                    else:
                        return self.layer(x)

            input_data = tree.map_structure(
                lambda x: ops.convert_to_numpy(x), input_data
            )
            output_data = tree.map_structure(
                lambda x: ops.convert_to_numpy(x), output_data
            )
            model = TestModel(layer)
            # Temporarily disable jit compilation on torch backend.
            jit_compile = keras.config.backend() != "torch"
            model.compile(optimizer="sgd", loss="mse", jit_compile=jit_compile)
            model.fit(input_data, output_data, verbose=0)

        # Build test.
        layer = cls(**init_kwargs)
        if isinstance(input_data, dict):
            shapes = {k + "_shape": v.shape for k, v in input_data.items()}
            if isinstance(layer, FeaturePyramid):
                layer.build(shapes)
            else:
                layer.build(**shapes)
        else:
            layer.build(input_data.shape)
        run_build_asserts(layer)

        # Symbolic call test.
        keras_tensor_inputs = tree.map_structure(
            lambda x: keras.KerasTensor(x.shape, x.dtype), input_data
        )
        layer = cls(**init_kwargs)
        if isinstance(keras_tensor_inputs, dict):
            if isinstance(layer, FeaturePyramid):
                keras_tensor_outputs = layer(keras_tensor_inputs)
            else:
                keras_tensor_outputs = layer(**keras_tensor_inputs)
        else:
            keras_tensor_outputs = layer(keras_tensor_inputs)
        run_build_asserts(layer)
        run_output_asserts(layer, keras_tensor_outputs)

        # Eager call test and compiled training test.
        layer = cls(**init_kwargs)
        if isinstance(input_data, dict):
            if isinstance(layer, FeaturePyramid):
                output_data = layer(input_data)
            else:
                output_data = layer(**input_data)
        else:
            output_data = layer(input_data)
        run_output_asserts(layer, output_data, eager=True)

        if run_training_check:
            run_training_step(layer, input_data, output_data)

        if run_precision_checks:
            self.run_precision_test(cls, init_kwargs, input_data)

    def run_preprocessing_layer_test(
        self,
        cls,
        init_kwargs,
        input_data,
        expected_output=None,
        expected_detokenize_output=None,
    ):
        """Run basic tests for a preprocessing layer."""
        layer = cls(**init_kwargs)
        # Check serialization (without a full save).
        self.run_serialization_test(layer)

        ds = tf.data.Dataset.from_tensor_slices(input_data)

        # Run with direct call.
        if isinstance(input_data, tuple):
            # Mimic tf.data unpacking behavior for preprocessing layers.
            output = layer(*input_data)
        else:
            output = layer(input_data)

        # For tokenizers only, also check detokenize.
        if isinstance(layer, Tokenizer):
            if not expected_detokenize_output:
                expected_detokenize_output = input_data
            detokenize_output = layer.detokenize(output)
            self.assertAllEqual(detokenize_output, expected_detokenize_output)

        # Run with an unbatched dataset.
        output_ds = ds.map(layer).ragged_batch(1_000)
        self.assertAllClose(output, output_ds.get_single_element())

        # Run with a batched dataset.
        output_ds = ds.batch(1_000).map(layer)
        self.assertAllClose(output, output_ds.get_single_element())

        if expected_output:
            self.assertAllClose(output, expected_output)

    def run_preprocessor_test(
        self,
        cls,
        init_kwargs,
        input_data,
        expected_output=None,
        expected_detokenize_output=None,
        token_id_key="token_ids",
    ):
        """Run basic tests for a Model Preprocessor layer."""
        self.run_preprocessing_layer_test(
            cls,
            init_kwargs,
            input_data,
            expected_output=expected_output,
            expected_detokenize_output=expected_detokenize_output,
        )

        layer = cls(**self.init_kwargs)
        if isinstance(input_data, tuple):
            output = layer(*input_data)
        else:
            output = layer(input_data)
        output, _, _ = keras.utils.unpack_x_y_sample_weight(output)
        shape = ops.shape(output[token_id_key])
        self.assertEqual(shape[-1], layer.sequence_length)
        # Update the sequence length.
        layer.sequence_length = 17
        if isinstance(input_data, tuple):
            output = layer(*input_data)
        else:
            output = layer(input_data)
        output, _, _ = keras.utils.unpack_x_y_sample_weight(output)
        shape = ops.shape(output[token_id_key])
        self.assertEqual(shape[-1], 17)

    def run_serialization_test(self, instance):
        """Check idempotency of serialize/deserialize.

        Not this is a much faster test than saving."""
        run_dir_test = (
            not keras.config.backend() == "tensorflow"
            or not isinstance(instance, Tokenizer)
        )
        # get_config roundtrip
        cls = instance.__class__
        cfg = instance.get_config()
        cfg_json = json.dumps(cfg, sort_keys=True, indent=4)
        ref_dir = dir(instance)[:]
        revived_instance = cls.from_config(cfg)
        revived_cfg = revived_instance.get_config()
        revived_cfg_json = json.dumps(revived_cfg, sort_keys=True, indent=4)
        self.assertEqual(cfg_json, revived_cfg_json)
        if run_dir_test:
            self.assertEqual(set(ref_dir), set(dir(revived_instance)))

        # serialization roundtrip
        serialized = keras.saving.serialize_keras_object(instance)
        serialized_json = json.dumps(serialized, sort_keys=True, indent=4)
        revived_instance = keras.saving.deserialize_keras_object(
            json.loads(serialized_json)
        )
        revived_cfg = revived_instance.get_config()
        revived_cfg_json = json.dumps(revived_cfg, sort_keys=True, indent=4)
        self.assertEqual(cfg_json, revived_cfg_json)
        if run_dir_test:
            new_dir = dir(revived_instance)[:]
            for lst in [ref_dir, new_dir]:
                if "__annotations__" in lst:
                    lst.remove("__annotations__")
            self.assertEqual(set(ref_dir), set(new_dir))

    def run_precision_test(self, cls, init_kwargs, input_data):
        # Never test mixed precision on torch CPU. Torch lacks support.
        if keras.config.backend() == "torch":
            import torch

            if not torch.cuda.is_available():
                return

        for policy in ["mixed_float16", "mixed_bfloat16", "bfloat16"]:
            policy = keras.mixed_precision.Policy(policy)
            # Ensure the correct `dtype` is set for sublayers or submodels in
            # `init_kwargs`.
            original_init_kwargs = init_kwargs.copy()
            for k, v in init_kwargs.items():
                if isinstance(v, keras.Layer):
                    config = v.get_config()
                    config["dtype"] = policy
                    init_kwargs[k] = v.__class__.from_config(config)
            layer = cls(**{**init_kwargs, "dtype": policy})
            if isinstance(layer, keras.Model):
                output_data = layer(input_data)
                output_spec = layer.compute_output_spec(input_data)
            elif isinstance(input_data, dict):
                if isinstance(layer, FeaturePyramid):
                    output_data = layer(input_data)
                    output_spec = layer.compute_output_spec(input_data)
                else:
                    output_data = layer(**input_data)
                    output_spec = layer.compute_output_spec(**input_data)
            else:
                output_data = layer(input_data)
                output_spec = layer.compute_output_spec(input_data)
            for tensor in tree.flatten(output_data):
                if is_float_dtype(tensor.dtype):
                    self.assertDTypeEqual(tensor, policy.compute_dtype)
            for spec in tree.flatten(output_spec):
                if is_float_dtype(spec.dtype):
                    self.assertDTypeEqual(spec, policy.compute_dtype)
            for weight in layer.weights:
                if is_float_dtype(weight.dtype):
                    self.assertDTypeEqual(weight, policy.variable_dtype)
            for sublayer in layer._flatten_layers():
                if isinstance(sublayer, keras.layers.Softmax):
                    continue
                if isinstance(sublayer, keras.layers.InputLayer):
                    continue
                self.assertEqual(policy.compute_dtype, sublayer.compute_dtype)
                self.assertEqual(policy.variable_dtype, sublayer.variable_dtype)
            # Restore `init_kwargs`.
            init_kwargs = original_init_kwargs

    def run_quantization_test(self, instance, cls, init_kwargs, input_data):
        # TODO: revert the following if. This works around a torch
        # quantization failure in `MultiHeadAttention` with Keras 3.7.
        if keras.config.backend() == "torch":
            return

        def _get_supported_layers(mode):
            supported_layers = [keras.layers.Dense, keras.layers.EinsumDense]
            if mode == "int8":
                supported_layers.append(keras.layers.Embedding)
                supported_layers.append(ReversibleEmbedding)
            return supported_layers

        for mode in ["int8", "float8"]:
            # Manually configure DTypePolicyMap to avoid intensive computation
            # in `Model.quantize`.
            policy_map = keras.dtype_policies.DTypePolicyMap("float32")
            for layer in instance._flatten_layers():
                if type(layer) in _get_supported_layers(mode):
                    policy_map[layer.path] = keras.dtype_policies.get(
                        f"{mode}_from_float32"
                    )
            # Ensure the correct `dtype` is set for sublayers or submodels in
            # `init_kwargs`.
            original_init_kwargs = init_kwargs.copy()
            for k, v in init_kwargs.items():
                if isinstance(v, keras.Layer):
                    config = v.get_config()
                    config["dtype"] = policy_map
                    init_kwargs[k] = v.__class__.from_config(config)
            # Instantiate the layer.
            model = cls(**{**init_kwargs, "dtype": policy_map})
            # Call layer eagerly.
            if isinstance(model, keras.Model):
                _ = model(input_data)
            elif isinstance(input_data, dict):
                _ = model(**input_data)
            else:
                _ = model(input_data)
            # Verify sublayer's dtype policy.
            for sublayer in model._flatten_layers():
                if type(sublayer) in _get_supported_layers(mode):
                    self.assertEqual(mode, sublayer.quantization_mode)
            # `get_config` roundtrip.
            cfg = model.get_config()
            revived_model = cls.from_config(cfg)
            revived_cfg = revived_model.get_config()
            self.assertEqual(cfg, revived_cfg)
            # Check weights loading.
            weights = model.get_weights()
            revived_model.set_weights(weights)
            # Restore `init_kwargs`.
            init_kwargs = original_init_kwargs

    def run_model_saving_test(
        self,
        cls,
        init_kwargs,
        input_data,
        atol=0.000001,
        rtol=0.000001,
    ):
        """Save and load a model from disk and assert output is unchanged."""
        model = cls(**init_kwargs)
        model_output = model(input_data)
        path = os.path.join(self.get_temp_dir(), "model.keras")
        model.save(path, save_format="keras_v3")
        restored_model = keras.models.load_model(path)

        # Check we got the real object back.
        self.assertIsInstance(restored_model, cls)

        # Check that output matches.
        restored_output = restored_model(input_data)
        self.assertAllClose(model_output, restored_output, atol=atol, rtol=rtol)

    def run_backbone_test(
        self,
        cls,
        init_kwargs,
        input_data,
        expected_output_shape,
        variable_length_data=None,
        run_mixed_precision_check=True,
        run_quantization_check=True,
    ):
        """Run basic tests for a backbone, including compilation."""
        backbone = cls(**init_kwargs)
        # Check serialization (without a full save).
        self.run_serialization_test(backbone)

        # Call model eagerly.
        output = backbone(input_data)
        if isinstance(expected_output_shape, dict):
            for key in expected_output_shape:
                self.assertEqual(output[key].shape, expected_output_shape[key])
        else:
            self.assertEqual(output.shape, expected_output_shape)
        if backbone.token_embedding is not None:
            # Check we can embed tokens eagerly.
            output = backbone.token_embedding(ops.zeros((2, 3), dtype="int32"))

            # Check variable length sequences.
            if variable_length_data is None:
                # If no variable length data passed, assume the second axis of
                # all inputs is our sequence axis and create it ourselves.
                variable_length_data = [
                    tree.map_structure(
                        lambda x: x[:, :seq_length, ...], input_data
                    )
                    for seq_length in (2, 3, 4)
                ]
            for batch in variable_length_data:
                backbone(batch)

        # Check compiled predict function.
        backbone.predict(input_data)
        # Convert to numpy first, torch GPU tensor -> tf.data will error.
        numpy_data = tree.map_structure(ops.convert_to_numpy, input_data)
        # Create a dataset.
        input_dataset = tf.data.Dataset.from_tensor_slices(numpy_data).batch(2)
        backbone.predict(input_dataset)

        # Check name maps to classname.
        name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", cls.__name__)
        name = re.sub("([a-z])([A-Z])", r"\1_\2", name).lower()
        self.assertRegex(backbone.name, name)

        # Check mixed precision.
        if run_mixed_precision_check:
            self.run_precision_test(cls, init_kwargs, input_data)

        # Check quantization.
        if run_quantization_check:
            self.run_quantization_test(backbone, cls, init_kwargs, input_data)

    def run_vision_backbone_test(
        self,
        cls,
        init_kwargs,
        input_data,
        expected_output_shape,
        expected_pyramid_output_keys=None,
        expected_pyramid_image_sizes=None,
        variable_length_data=None,
        run_mixed_precision_check=True,
        run_quantization_check=True,
        run_data_format_check=True,
    ):
        """Run basic tests for a vision backbone, including compilation."""
        can_run_data_format_check = True
        if (
            keras.config.backend() == "tensorflow"
            and not tf.config.list_physical_devices("GPU")
        ):
            # Never test the "channels_first" format on tensorflow CPU.
            # Tensorflow lacks support for "channels_first" convolution.
            can_run_data_format_check = False

        ori_data_format = keras.config.image_data_format()
        keras.config.set_image_data_format("channels_last")
        self.run_backbone_test(
            cls=cls,
            init_kwargs=init_kwargs,
            input_data=input_data,
            expected_output_shape=expected_output_shape,
            variable_length_data=variable_length_data,
            run_mixed_precision_check=run_mixed_precision_check,
            run_quantization_check=run_quantization_check,
        )

        if expected_pyramid_output_keys:
            backbone = cls(**init_kwargs)
            model = keras.models.Model(
                backbone.inputs, backbone.pyramid_outputs
            )
            output_data = model(input_data)

            self.assertIsInstance(output_data, dict)
            self.assertEqual(
                list(output_data.keys()), list(backbone.pyramid_outputs.keys())
            )
            self.assertEqual(
                list(output_data.keys()), expected_pyramid_output_keys
            )
            # check height and width of each level.
            for i, (k, v) in enumerate(output_data.items()):
                self.assertEqual(
                    tuple(v.shape[1:3]), expected_pyramid_image_sizes[i]
                )

        # Check data_format. We assume that `input_data` is in "channels_last"
        # format.
        if run_data_format_check and can_run_data_format_check:
            keras.config.set_image_data_format("channels_first")
            input_data_shape = ops.shape(input_data)
            if len(input_data_shape) == 3:
                input_data = ops.transpose(input_data, axes=(2, 0, 1))
            elif len(input_data_shape) == 4:
                input_data = ops.transpose(input_data, axes=(0, 3, 1, 2))
            if len(expected_output_shape) == 3:
                x = expected_output_shape
                expected_output_shape = (x[0], x[2], x[1])
            elif len(expected_output_shape) == 4:
                x = expected_output_shape
                expected_output_shape = (x[0], x[3], x[1], x[2])
            if "image_shape" in init_kwargs:
                init_kwargs = init_kwargs.copy()
                init_kwargs["image_shape"] = tuple(
                    reversed(init_kwargs["image_shape"])
                )
            self.run_backbone_test(
                cls=cls,
                init_kwargs=init_kwargs,
                input_data=input_data,
                expected_output_shape=expected_output_shape,
                variable_length_data=variable_length_data,
                run_mixed_precision_check=run_mixed_precision_check,
                run_quantization_check=run_quantization_check,
            )

        # Restore the original `image_data_format`.
        keras.config.set_image_data_format(ori_data_format)

    def run_task_test(
        self,
        cls,
        init_kwargs,
        train_data,
        expected_output_shape=None,
        batch_size=2,
    ):
        """Run basic tests for a backbone, including compilation."""
        task = cls(**init_kwargs)
        # Check serialization (without a full save).
        self.run_serialization_test(task)
        preprocessor = task.preprocessor
        ds = tf.data.Dataset.from_tensor_slices(train_data).batch(batch_size)
        x, y, sw = keras.utils.unpack_x_y_sample_weight(train_data)

        # Test: the tree struct output by the
        # preprocessor must match what model expects.
        preprocessed_data = preprocessor(*train_data)[0]
        tree.assert_same_structure(
            preprocessed_data,
            task._inputs_struct,
            check_types=False,
        )

        # Test predict.
        output = task.predict(x)
        if expected_output_shape is not None:
            output_shape = tree.map_structure(lambda x: x.shape, output)
            self.assertAllClose(output_shape, expected_output_shape)
        # With a dataset.
        output_ds = task.predict(ds)
        self.assertAllClose(output, output_ds)
        # With split preprocessing.
        task.preprocessor = None
        output_split = task.predict(ds.map(preprocessor))
        task.preprocessor = preprocessor
        self.assertAllClose(output, output_split)

        # Test fit.
        task.fit(x, y, sample_weight=sw)
        # With a dataset.
        task.fit(ds)
        # With split preprocessing.
        task.preprocessor = None
        task.fit(ds.map(preprocessor))
        task.preprocessor = preprocessor
        # Turn off default compilation, should error during `fit()`.
        task = cls(**init_kwargs, compile=False)
        with self.assertRaisesRegex(ValueError, "You must call `compile"):
            task.fit(ds)

    def run_preset_test(
        self,
        cls,
        preset,
        input_data,
        init_kwargs={},
        expected_output=None,
        expected_output_shape=None,
        expected_partial_output=None,
        expected_labels=None,
    ):
        """Run instantiation and a forward pass for a preset."""
        with self.assertRaises(Exception):
            cls.from_preset("clowntown", **init_kwargs)

        instance = cls.from_preset(preset, **init_kwargs)

        if isinstance(input_data, tuple):
            # Mimic tf.data unpacking behavior for preprocessing layers.
            output = instance(*input_data)
        else:
            output = instance(input_data)

        if isinstance(instance, keras.Model):
            instance = cls.from_preset(
                preset, load_weights=False, **init_kwargs
            )
            instance(input_data)

        if expected_output is not None:
            self.assertAllClose(output, expected_output)

        if expected_output_shape is not None:
            output_shape = tree.map_structure(lambda x: x.shape, output)
            self.assertAllClose(output_shape, expected_output_shape)

        if expected_partial_output is not None:
            # Allow passing a partial output snippet of the last dimension.
            # We want check stability, but the full output would be too long.
            def compare(actual, expected):
                expected = ops.convert_to_numpy(expected)
                self.assertEqual(len(expected.shape), 1)
                actual = ops.reshape(actual, (-1,))[: expected.shape[0]]
                self.assertAllClose(actual, expected, atol=0.01, rtol=0.01)

            tree.map_structure(compare, output, expected_partial_output)

        if expected_labels is not None:
            output = ops.argmax(output, axis=-1)
            self.assertAllEqual(output, expected_labels)

    def get_test_data_dir(self):
        return str(pathlib.Path(__file__).parent / "test_data")

    def load_test_image(self, target_size=None):
        # From https://commons.wikimedia.org/wiki/File:California_quail.jpg
        path = os.path.join(self.get_test_data_dir(), "test_image.jpg")
        img = keras.utils.load_img(
            path, target_size=target_size, keep_aspect_ratio=True
        )
        return np.array(img)



import math

import keras
from keras import ops

from keras_hub.src.utils.keras_utils import standardize_data_format


class Conv2DMultiHeadAttention(keras.layers.Layer):
    """A MultiHeadAttention layer utilizing `Conv2D` and `GroupNormalization`.

    Args:
        filters: int. The number of the filters for the convolutional layers.
        groups: int. The number of the groups for the group normalization
            layers. Defaults to `32`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.
    """

    def __init__(self, filters, groups=32, data_format=None, **kwargs):
        super().__init__(**kwargs)
        data_format = standardize_data_format(data_format)
        channel_axis = -1 if data_format == "channels_last" else 1
        self.filters = int(filters)
        self.groups = int(groups)
        self._inverse_sqrt_filters = 1.0 / math.sqrt(float(filters))
        self.data_format = data_format

        self.group_norm = keras.layers.GroupNormalization(
            groups=groups,
            axis=channel_axis,
            epsilon=1e-6,
            dtype=self.dtype_policy,
            name="group_norm",
        )
        self.query_conv2d = keras.layers.Conv2D(
            filters,
            1,
            1,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="query_conv2d",
        )
        self.key_conv2d = keras.layers.Conv2D(
            filters,
            1,
            1,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="key_conv2d",
        )
        self.value_conv2d = keras.layers.Conv2D(
            filters,
            1,
            1,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="value_conv2d",
        )
        self.softmax = keras.layers.Softmax(dtype="float32")
        self.output_conv2d = keras.layers.Conv2D(
            filters,
            1,
            1,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="output_conv2d",
        )

    def build(self, input_shape):
        self.group_norm.build(input_shape)
        self.query_conv2d.build(input_shape)
        self.key_conv2d.build(input_shape)
        self.value_conv2d.build(input_shape)
        self.output_conv2d.build(input_shape)

    def call(self, inputs, training=None):
        x = self.group_norm(inputs, training=training)
        query = self.query_conv2d(x, training=training)
        key = self.key_conv2d(x, training=training)
        value = self.value_conv2d(x, training=training)

        if self.data_format == "channels_first":
            query = ops.transpose(query, (0, 2, 3, 1))
            key = ops.transpose(key, (0, 2, 3, 1))
            value = ops.transpose(value, (0, 2, 3, 1))
        shape = ops.shape(inputs)
        b = shape[0]
        query = ops.reshape(query, (b, -1, self.filters))
        key = ops.reshape(key, (b, -1, self.filters))
        value = ops.reshape(value, (b, -1, self.filters))

        # Compute attention.
        query = ops.multiply(
            query, ops.cast(self._inverse_sqrt_filters, query.dtype)
        )
        # [B, H0 * W0, C], [B, H1 * W1, C] -> [B, H0 * W0, H1 * W1]
        attention_scores = ops.einsum("abc,adc->abd", query, key)
        attention_scores = ops.cast(
            self.softmax(attention_scores), self.compute_dtype
        )
        # [B, H2 * W2, C], [B, H0 * W0, H1 * W1] -> [B, H1 * W1 ,C]
        attention_output = ops.einsum("abc,adb->adc", value, attention_scores)
        x = ops.reshape(attention_output, shape)

        x = self.output_conv2d(x, training=training)
        if self.data_format == "channels_first":
            x = ops.transpose(x, (0, 3, 1, 2))
        x = ops.add(x, inputs)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "filters": self.filters,
                "groups": self.groups,
            }
        )
        return config

    def compute_output_shape(self, input_shape):
        return input_shape


class ResNetBlock(keras.layers.Layer):
    """A ResNet block utilizing `GroupNormalization` and SiLU activation.

    Args:
        filters: The number of filters in the block.
        has_residual_projection: Whether to add a projection layer for the
            residual connection. Defaults to `False`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.
    """

    def __init__(
        self,
        filters,
        has_residual_projection=False,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        data_format = standardize_data_format(data_format)
        channel_axis = -1 if data_format == "channels_last" else 1
        self.filters = int(filters)
        self.has_residual_projection = bool(has_residual_projection)

        # === Layers ===
        self.norm1 = keras.layers.GroupNormalization(
            groups=32,
            axis=channel_axis,
            epsilon=1e-6,
            dtype=self.dtype_policy,
            name="norm1",
        )
        self.act1 = keras.layers.Activation("silu", dtype=self.dtype_policy)
        self.conv1 = keras.layers.Conv2D(
            filters,
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="conv1",
        )
        self.norm2 = keras.layers.GroupNormalization(
            groups=32,
            axis=channel_axis,
            epsilon=1e-6,
            dtype=self.dtype_policy,
            name="norm2",
        )
        self.act2 = keras.layers.Activation("silu", dtype=self.dtype_policy)
        self.conv2 = keras.layers.Conv2D(
            filters,
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="conv2",
        )
        if self.has_residual_projection:
            self.residual_projection = keras.layers.Conv2D(
                filters,
                1,
                1,
                data_format=data_format,
                dtype=self.dtype_policy,
                name="residual_projection",
            )
        self.add = keras.layers.Add(dtype=self.dtype_policy)

    def build(self, input_shape):
        residual_shape = list(input_shape)
        self.norm1.build(input_shape)
        self.act1.build(input_shape)
        self.conv1.build(input_shape)
        input_shape = self.conv1.compute_output_shape(input_shape)
        self.norm2.build(input_shape)
        self.act2.build(input_shape)
        self.conv2.build(input_shape)
        input_shape = self.conv2.compute_output_shape(input_shape)
        if self.has_residual_projection:
            self.residual_projection.build(residual_shape)
        self.add.build([input_shape, input_shape])

    def call(self, inputs, training=None):
        x = inputs
        residual = x
        x = self.norm1(x, training=training)
        x = self.act1(x, training=training)
        x = self.conv1(x, training=training)
        x = self.norm2(x, training=training)
        x = self.act2(x, training=training)
        x = self.conv2(x, training=training)
        if self.has_residual_projection:
            residual = self.residual_projection(residual, training=training)
        x = self.add([residual, x])
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "filters": self.filters,
                "has_residual_projection": self.has_residual_projection,
            }
        )
        return config

    def compute_output_shape(self, input_shape):
        outputs_shape = list(input_shape)
        if self.has_residual_projection:
            outputs_shape = self.residual_projection.compute_output_shape(
                outputs_shape
            )
        return outputs_shape


class VAEEncoder(keras.layers.Layer):
    """The encoder layer of VAE.

    Args:
        stackwise_num_filters: list of ints. The number of filters for each
            stack.
        stackwise_num_blocks: list of ints. The number of blocks for each stack.
        output_channels: int. The number of channels in the output. Defaults to
            `32`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.
    """

    def __init__(
        self,
        stackwise_num_filters,
        stackwise_num_blocks,
        output_channels=32,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        data_format = standardize_data_format(data_format)
        channel_axis = -1 if data_format == "channels_last" else 1
        self.stackwise_num_filters = stackwise_num_filters
        self.stackwise_num_blocks = stackwise_num_blocks
        self.output_channels = int(output_channels)
        self.data_format = data_format

        # === Layers ===
        self.input_projection = keras.layers.Conv2D(
            stackwise_num_filters[0],
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="input_projection",
        )

        # Blocks.
        input_filters = stackwise_num_filters[0]
        self.blocks = []
        self.downsamples = []
        for i, filters in enumerate(stackwise_num_filters):
            for j in range(stackwise_num_blocks[i]):
                self.blocks.append(
                    ResNetBlock(
                        filters,
                        has_residual_projection=input_filters != filters,
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"block_{i}_{j}",
                    )
                )
                input_filters = filters
            # No downsample in the last block.
            if i != len(stackwise_num_filters) - 1:
                self.downsamples.append(
                    keras.layers.ZeroPadding2D(
                        padding=((0, 1), (0, 1)),
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"downsample_{i}_pad",
                    )
                )
                self.downsamples.append(
                    keras.layers.Conv2D(
                        filters,
                        3,
                        2,
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"downsample_{i}_conv",
                    )
                )

        # Mid block.
        self.mid_block_0 = ResNetBlock(
            stackwise_num_filters[-1],
            has_residual_projection=False,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_block_0",
        )
        self.mid_attention = Conv2DMultiHeadAttention(
            stackwise_num_filters[-1],
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_attention",
        )
        self.mid_block_1 = ResNetBlock(
            stackwise_num_filters[-1],
            has_residual_projection=False,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_block_1",
        )

        # Output layers.
        self.output_norm = keras.layers.GroupNormalization(
            groups=32,
            axis=channel_axis,
            epsilon=1e-6,
            dtype=self.dtype_policy,
            name="output_norm",
        )
        self.output_act = keras.layers.Activation(
            "swish", dtype=self.dtype_policy
        )
        self.output_projection = keras.layers.Conv2D(
            output_channels,
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="output_projection",
        )

    def build(self, input_shape):
        self.input_projection.build(input_shape)
        input_shape = self.input_projection.compute_output_shape(input_shape)
        blocks_idx = 0
        downsamples_idx = 0
        for i, _ in enumerate(self.stackwise_num_filters):
            for _ in range(self.stackwise_num_blocks[i]):
                self.blocks[blocks_idx].build(input_shape)
                input_shape = self.blocks[blocks_idx].compute_output_shape(
                    input_shape
                )
                blocks_idx += 1
            if i != len(self.stackwise_num_filters) - 1:
                self.downsamples[downsamples_idx].build(input_shape)
                input_shape = self.downsamples[
                    downsamples_idx
                ].compute_output_shape(input_shape)
                downsamples_idx += 1
                self.downsamples[downsamples_idx].build(input_shape)
                input_shape = self.downsamples[
                    downsamples_idx
                ].compute_output_shape(input_shape)
                downsamples_idx += 1
        self.mid_block_0.build(input_shape)
        input_shape = self.mid_block_0.compute_output_shape(input_shape)
        self.mid_attention.build(input_shape)
        input_shape = self.mid_attention.compute_output_shape(input_shape)
        self.mid_block_1.build(input_shape)
        input_shape = self.mid_block_1.compute_output_shape(input_shape)
        self.output_norm.build(input_shape)
        self.output_act.build(input_shape)
        self.output_projection.build(input_shape)

    def call(self, inputs, training=None):
        x = inputs
        x = self.input_projection(x, training=training)
        blocks_idx = 0
        upsamples_idx = 0
        for i, _ in enumerate(self.stackwise_num_filters):
            for _ in range(self.stackwise_num_blocks[i]):
                x = self.blocks[blocks_idx](x, training=training)
                blocks_idx += 1
            if i != len(self.stackwise_num_filters) - 1:
                x = self.downsamples[upsamples_idx](x, training=training)
                x = self.downsamples[upsamples_idx + 1](x, training=training)
                upsamples_idx += 2
        x = self.mid_block_0(x, training=training)
        x = self.mid_attention(x, training=training)
        x = self.mid_block_1(x, training=training)
        x = self.output_norm(x, training=training)
        x = self.output_act(x, training=training)
        x = self.output_projection(x, training=training)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "stackwise_num_filters": self.stackwise_num_filters,
                "stackwise_num_blocks": self.stackwise_num_blocks,
                "output_channels": self.output_channels,
            }
        )
        return config

    def compute_output_shape(self, input_shape):
        if self.data_format == "channels_last":
            h_axis, w_axis, c_axis = 1, 2, 3
        else:
            c_axis, h_axis, w_axis = 1, 2, 3
        scale_factor = 2 ** (len(self.stackwise_num_filters) - 1)
        outputs_shape = list(input_shape)
        if (
            outputs_shape[h_axis] is not None
            and outputs_shape[w_axis] is not None
        ):
            outputs_shape[h_axis] = outputs_shape[h_axis] // scale_factor
            outputs_shape[w_axis] = outputs_shape[w_axis] // scale_factor
        outputs_shape[c_axis] = self.output_channels
        return outputs_shape


class VAEDecoder(keras.layers.Layer):
    """The decoder layer of VAE.

    Args:
        stackwise_num_filters: list of ints. The number of filters for each
            stack.
        stackwise_num_blocks: list of ints. The number of blocks for each stack.
        output_channels: int. The number of channels in the output. Defaults to
            `3`.
        data_format: `None` or str. If specified, either `"channels_last"` or
            `"channels_first"`. The ordering of the dimensions in the
            inputs. `"channels_last"` corresponds to inputs with shape
            `(batch_size, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch_size, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.
    """

    def __init__(
        self,
        stackwise_num_filters,
        stackwise_num_blocks,
        output_channels=3,
        data_format=None,
        **kwargs,
    ):
        super().__init__(**kwargs)
        data_format = standardize_data_format(data_format)
        channel_axis = -1 if data_format == "channels_last" else 1
        self.stackwise_num_filters = stackwise_num_filters
        self.stackwise_num_blocks = stackwise_num_blocks
        self.output_channels = int(output_channels)
        self.data_format = data_format

        # === Layers ===
        self.input_projection = keras.layers.Conv2D(
            stackwise_num_filters[0],
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="input_projection",
        )

        # Mid block.
        self.mid_block_0 = ResNetBlock(
            stackwise_num_filters[0],
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_block_0",
        )
        self.mid_attention = Conv2DMultiHeadAttention(
            stackwise_num_filters[0],
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_attention",
        )
        self.mid_block_1 = ResNetBlock(
            stackwise_num_filters[0],
            data_format=data_format,
            dtype=self.dtype_policy,
            name="mid_block_1",
        )

        # Blocks.
        input_filters = stackwise_num_filters[0]
        self.blocks = []
        self.upsamples = []
        for i, filters in enumerate(stackwise_num_filters):
            for j in range(stackwise_num_blocks[i]):
                self.blocks.append(
                    ResNetBlock(
                        filters,
                        has_residual_projection=input_filters != filters,
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"block_{i}_{j}",
                    )
                )
                input_filters = filters
            # No upsample in the last block.
            if i != len(stackwise_num_filters) - 1:
                self.upsamples.append(
                    keras.layers.UpSampling2D(
                        2,
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"upsample_{i}",
                    )
                )
                self.upsamples.append(
                    keras.layers.Conv2D(
                        filters,
                        3,
                        1,
                        padding="same",
                        data_format=data_format,
                        dtype=self.dtype_policy,
                        name=f"upsample_{i}_conv",
                    )
                )

        # Output layers.
        self.output_norm = keras.layers.GroupNormalization(
            groups=32,
            axis=channel_axis,
            epsilon=1e-6,
            dtype=self.dtype_policy,
            name="output_norm",
        )
        self.output_act = keras.layers.Activation(
            "swish", dtype=self.dtype_policy
        )
        self.output_projection = keras.layers.Conv2D(
            output_channels,
            3,
            1,
            padding="same",
            data_format=data_format,
            dtype=self.dtype_policy,
            name="output_projection",
        )

    def build(self, input_shape):
        self.input_projection.build(input_shape)
        input_shape = self.input_projection.compute_output_shape(input_shape)
        self.mid_block_0.build(input_shape)
        input_shape = self.mid_block_0.compute_output_shape(input_shape)
        self.mid_attention.build(input_shape)
        input_shape = self.mid_attention.compute_output_shape(input_shape)
        self.mid_block_1.build(input_shape)
        input_shape = self.mid_block_1.compute_output_shape(input_shape)
        blocks_idx = 0
        upsamples_idx = 0
        for i, _ in enumerate(self.stackwise_num_filters):
            for _ in range(self.stackwise_num_blocks[i]):
                self.blocks[blocks_idx].build(input_shape)
                input_shape = self.blocks[blocks_idx].compute_output_shape(
                    input_shape
                )
                blocks_idx += 1
            if i != len(self.stackwise_num_filters) - 1:
                self.upsamples[upsamples_idx].build(input_shape)
                input_shape = self.upsamples[
                    upsamples_idx
                ].compute_output_shape(input_shape)
                self.upsamples[upsamples_idx + 1].build(input_shape)
                input_shape = self.upsamples[
                    upsamples_idx + 1
                ].compute_output_shape(input_shape)
                upsamples_idx += 2
        self.output_norm.build(input_shape)
        self.output_act.build(input_shape)
        self.output_projection.build(input_shape)

    def call(self, inputs, training=None):
        x = inputs
        x = self.input_projection(x, training=training)
        x = self.mid_block_0(x, training=training)
        x = self.mid_attention(x, training=training)
        x = self.mid_block_1(x, training=training)
        blocks_idx = 0
        upsamples_idx = 0
        for i, _ in enumerate(self.stackwise_num_filters):
            for _ in range(self.stackwise_num_blocks[i]):
                x = self.blocks[blocks_idx](x, training=training)
                blocks_idx += 1
            if i != len(self.stackwise_num_filters) - 1:
                x = self.upsamples[upsamples_idx](x, training=training)
                x = self.upsamples[upsamples_idx + 1](x, training=training)
                upsamples_idx += 2
        x = self.output_norm(x, training=training)
        x = self.output_act(x, training=training)
        x = self.output_projection(x, training=training)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "stackwise_num_filters": self.stackwise_num_filters,
                "stackwise_num_blocks": self.stackwise_num_blocks,
                "output_channels": self.output_channels,
            }
        )
        return config

    def compute_output_shape(self, input_shape):
        if self.data_format == "channels_last":
            h_axis, w_axis, c_axis = 1, 2, 3
        else:
            c_axis, h_axis, w_axis = 1, 2, 3
        scale_factor = 2 ** (len(self.stackwise_num_filters) - 1)
        outputs_shape = list(input_shape)
        if (
            outputs_shape[h_axis] is not None
            and outputs_shape[w_axis] is not None
        ):
            outputs_shape[h_axis] = outputs_shape[h_axis] * scale_factor
            outputs_shape[w_axis] = outputs_shape[w_axis] * scale_factor
        outputs_shape[c_axis] = self.output_channels
        return outputs_shape


class DiagonalGaussianDistributionSampler(keras.layers.Layer):
    """A sampler for a diagonal Gaussian distribution.

    This layer samples latent variables from a diagonal Gaussian distribution.

    Args:
        method: str. The method used to sample from the distribution. Available
            methods are `"sample"` and `"mode"`. `"sample"` draws from the
            distribution using both the mean and log variance. `"mode"` draws
            from the distribution using the mean only.
        axis: int. The axis along which to split the mean and log variance.
            Defaults to `-1`.
        seed: optional int. Used as a random seed.
        **kwargs: other keyword arguments passed to `keras.layers.Layer`,
            including `name`, `dtype` etc.
    """

    def __init__(self, method, axis=-1, seed=None, **kwargs):
        super().__init__(**kwargs)
        # TODO: Support `kl` and `nll` modes.
        valid_methods = ("sample", "mode")
        if method not in valid_methods:
            raise ValueError(
                f"Invalid method {method}. Valid methods are "
                f"{list(valid_methods)}."
            )
        self.method = method
        self.axis = axis
        self.seed = seed
        self.seed_generator = keras.random.SeedGenerator(seed)

    def call(self, inputs):
        x = inputs
        if self.method == "sample":
            x_mean, x_logvar = ops.split(x, 2, axis=self.axis)
            x_logvar = ops.clip(x_logvar, -30.0, 20.0)
            x_std = ops.exp(ops.multiply(0.5, x_logvar))
            sample = keras.random.normal(
                ops.shape(x_mean), dtype=x_mean.dtype, seed=self.seed_generator
            )
            x = ops.add(x_mean, ops.multiply(x_std, sample))
        else:
            x, _ = ops.split(x, 2, axis=self.axis)
        return x

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "axis": self.axis,
                "seed": self.seed,
            }
        )
        return config

    def compute_output_shape(self, input_shape):
        output_shape = list(input_shape)
        output_shape[self.axis] = output_shape[self.axis] // 2
        return output_shape



from keras_hub.src.models.bert.bert_backbone import BertBackbone
from keras_hub.src.models.bert.bert_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, BertBackbone)



from keras_hub.src.models.bloom.bloom_backbone import BloomBackbone
from keras_hub.src.models.bloom.bloom_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, BloomBackbone)



"""T5 model preset configurations."""

backbone_presets = {
    "t5_small_multi": {
        "metadata": {
            "description": (
                "8-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_small_multi/3",
    },
    "t5_1.1_small": {
        "metadata": {
            "description": (""),
            "params": 60511616,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_1.1_small/2",
    },
    "t5_base_multi": {
        "metadata": {
            "description": (
                "12-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_base_multi/3",
    },
    "t5_1.1_base": {
        "metadata": {
            "description": (""),
            "params": 247577856,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_1.1_base/2",
    },
    "t5_large_multi": {
        "metadata": {
            "description": (
                "24-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_large_multi/3",
    },
    "t5_1.1_large": {
        "metadata": {
            "description": (""),
            "params": 750251008,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_1.1_large/2",
    },
    "t5_1.1_xl": {
        "metadata": {
            "description": (""),
            "params": 2849757184,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_1.1_xl/2",
    },
    "t5_1.1_xxl": {
        "metadata": {
            "description": (""),
            "params": 11135332352,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/t5_1.1_xxl/2",
    },
    "flan_small_multi": {
        "metadata": {
            "description": (
                "8-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/flan_small_multi/3",
    },
    "flan_base_multi": {
        "metadata": {
            "description": (
                "12-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/flan_base_multi/3",
    },
    "flan_large_multi": {
        "metadata": {
            "description": (
                "24-layer T5 model. Trained on the Colossal Clean Crawled "
                "Corpus (C4)."
            ),
            "params": 0,
            "path": "t5",
        },
        "kaggle_handle": "kaggle://keras/t5/keras/flan_large_multi/3",
    },
}



import keras

from keras_hub.src.utils.tensor_utils import assert_tf_libs_installed


class PreprocessingLayer(keras.layers.Layer):
    """Preprocessing layer base class."""

    def __init__(self, **kwargs):
        assert_tf_libs_installed(self.__class__.__name__)
        super().__init__(**kwargs)
        # Don't convert inputs (we want tf tensors not backend tensors).
        self._convert_input_args = False
        # Allow raw inputs like python strings.
        self._allow_non_tensor_positional_args = True
        # Most pre-preprocessing has no build.
        if not hasattr(self, "build"):
            self.built = True

    def get_build_config(self):
        return None



from keras_hub.src.models.retinanet.retinanet_backbone import RetinaNetBackbone
from keras_hub.src.models.retinanet.retinanet_presets import backbone_presets
from keras_hub.src.utils.preset_utils import register_presets

register_presets(backbone_presets, RetinaNetBackbone)



"""GPT-2 model preset configurations."""

# Metadata for loading pretrained model weights.
backbone_presets = {
    "gpt2_base_en": {
        "metadata": {
            "description": (
                "12-layer GPT-2 model where case is maintained. "
                "Trained on WebText."
            ),
            "params": 124439808,
            "path": "gpt2",
        },
        "kaggle_handle": "kaggle://keras/gpt2/keras/gpt2_base_en/3",
    },
    "gpt2_medium_en": {
        "metadata": {
            "description": (
                "24-layer GPT-2 model where case is maintained. "
                "Trained on WebText."
            ),
            "params": 354823168,
            "path": "gpt2",
        },
        "kaggle_handle": "kaggle://keras/gpt2/keras/gpt2_medium_en/3",
    },
    "gpt2_large_en": {
        "metadata": {
            "description": (
                "36-layer GPT-2 model where case is maintained. "
                "Trained on WebText."
            ),
            "params": 774030080,
            "path": "gpt2",
        },
        "kaggle_handle": "kaggle://keras/gpt2/keras/gpt2_large_en/3",
    },
    "gpt2_extra_large_en": {
        "metadata": {
            "description": (
                "48-layer GPT-2 model where case is maintained. "
                "Trained on WebText."
            ),
            "params": 1557611200,
            "path": "gpt2",
        },
        "kaggle_handle": "kaggle://keras/gpt2/keras/gpt2_extra_large_en/3",
    },
    "gpt2_base_en_cnn_dailymail": {
        "metadata": {
            "description": (
                "12-layer GPT-2 model where case is maintained. "
                "Finetuned on the CNN/DailyMail summarization dataset."
            ),
            "params": 124439808,
            "path": "gpt2",
        },
        "kaggle_handle": "kaggle://keras/gpt2/keras/gpt2_base_en_cnn_dailymail/3",
    },
}



import math

import keras
import numpy as np
from keras import ops

from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.preprocessing_layer import (
    PreprocessingLayer,
)
from keras_hub.src.utils.keras_utils import standardize_data_format
from keras_hub.src.utils.preset_utils import builtin_presets
from keras_hub.src.utils.preset_utils import find_subclass
from keras_hub.src.utils.preset_utils import get_preset_loader
from keras_hub.src.utils.preset_utils import get_preset_saver
from keras_hub.src.utils.python_utils import classproperty
from keras_hub.src.utils.tensor_utils import check_bounding_box_support
from keras_hub.src.utils.tensor_utils import preprocessing_function


@keras_hub_export("keras_hub.layers.ImageConverter")
class ImageConverter(PreprocessingLayer):
    """Preprocess raw images into model ready inputs.

    This class converts from raw images to model ready inputs. This conversion
    proceeds in the following steps:

    1. Resize the image using to `image_size`. If `image_size` is `None`, this
       step will be skipped.
    2. Rescale the image by multiplying by `scale`, which can be either global
       or per channel. If `scale` is `None`, this step will be skipped.
    3. Offset the image by adding `offset`, which can be either global
       or per channel. If `offset` is `None`, this step will be skipped.

    The layer will take as input a raw image tensor in the channels last or
    channels first format, and output a preprocessed image input for modeling.
    This tensor can be batched (rank 4), or unbatched (rank 3).

    This layer can be used with the `from_preset()` constructor to load a layer
    that will rescale and resize an image for a specific pretrained model.
    Using the layer this way allows writing preprocessing code that does not
    need updating when switching between model checkpoints.

    Args:
        image_size: `(int, int)` tuple or `None`. The output size of the image,
            not including the channels axis. If `None`, the input will not be
            resized.
        scale: float, tuple of floats, or `None`. The scale to apply to the
            inputs. If `scale` is a single float, the entire input will be
            multiplied by `scale`. If `scale` is a tuple, it's assumed to
            contain per-channel scale value multiplied against each channel of
            the input images. If `scale` is `None`, no scaling is applied.
        offset: float, tuple of floats, or `None`. The offset to apply to the
            inputs. If `offset` is a single float, the entire input will be
            summed with `offset`. If `offset` is a tuple, it's assumed to
            contain per-channel offset value summed against each channel of the
            input images. If `offset` is `None`, no scaling is applied.
        crop_to_aspect_ratio: If `True`, resize the images without aspect
            ratio distortion. When the original aspect ratio differs
            from the target aspect ratio, the output image will be
            cropped so as to return the
            largest possible window in the image (of size `(height, width)`)
            that matches the target aspect ratio. By default
            (`crop_to_aspect_ratio=False`), aspect ratio may not be preserved.
        interpolation: String, the interpolation method.
            Supports `"bilinear"`, `"nearest"`, `"bicubic"`,
            `"lanczos3"`, `"lanczos5"`. Defaults to `"bilinear"`.
        bounding_box_format: A string specifying the format of the bounding
            boxes, one of `"xyxy"`, `"rel_xyxy"`, `"xywh"`, `"center_xywh"`,
            `"yxyx"`, `"rel_yxyx"`. Specifies the format of the bounding boxes
            which will be resized to `image_size` along with the image. To pass
            bounding boxed to this layer, pass a dict with keys `"images"` and
            `"bounding_boxes"` when calling the layer.
        data_format: String, either `"channels_last"` or `"channels_first"`.
            The ordering of the dimensions in the inputs. `"channels_last"`
            corresponds to inputs with shape `(batch, height, width, channels)`
            while `"channels_first"` corresponds to inputs with shape
            `(batch, channels, height, width)`. It defaults to the
            `image_data_format` value found in your Keras config file at
            `~/.keras/keras.json`. If you never set it, then it will be
            `"channels_last"`.

    Examples:
    ```python
    # Resize raw images and scale them to [0, 1].
    converter = keras_hub.layers.ImageConverter(
        image_size=(128, 128),
        scale=1. / 255,
    )
    converter(np.random.randint(0, 256, size=(2, 512, 512, 3)))

    # Resize images to the specific size needed for a PaliGemma preset.
    converter = keras_hub.layers.ImageConverter.from_preset(
        "pali_gemma_3b_224"
    )
    converter(np.random.randint(0, 256, size=(2, 512, 512, 3)))
    ```
    """

    backbone_cls = None

    def __init__(
        self,
        image_size=None,
        scale=None,
        offset=None,
        crop_to_aspect_ratio=True,
        pad_to_aspect_ratio=False,
        interpolation="bilinear",
        bounding_box_format="yxyx",
        data_format=None,
        **kwargs,
    ):
        # TODO: old arg names. Delete this block after resaving Kaggle assets.
        if "height" in kwargs and "width" in kwargs:
            image_size = (kwargs.pop("height"), kwargs.pop("width"))
        if "variance" in kwargs and "mean" in kwargs:
            std = [math.sqrt(v) for v in kwargs.pop("variance")]
            scale = [scale / s for s in std]
            offset = [-m / s for m, s in zip(kwargs.pop("mean"), std)]

        super().__init__(**kwargs)

        if crop_to_aspect_ratio and pad_to_aspect_ratio:
            raise ValueError(
                "Only one of 'crop_to_aspect_ratio' or 'pad_to_aspect_ratio' "
                "can be True."
            )

        # Create the `Resizing` layer here even if it's not being used. That
        # allows us to make `image_size` a settable property.
        resizing_kwargs = {}
        if check_bounding_box_support():
            resizing_kwargs["bounding_box_format"] = bounding_box_format
        self.resizing = keras.layers.Resizing(
            height=image_size[0] if image_size else None,
            width=image_size[1] if image_size else None,
            crop_to_aspect_ratio=crop_to_aspect_ratio,
            pad_to_aspect_ratio=pad_to_aspect_ratio,
            interpolation=interpolation,
            data_format=data_format,
            dtype=self.dtype_policy,
            name="resizing",
            **resizing_kwargs,
        )
        self.scale = scale
        self.offset = offset
        self.crop_to_aspect_ratio = crop_to_aspect_ratio
        self.pad_to_aspect_ratio = pad_to_aspect_ratio
        self.interpolation = interpolation
        self.bounding_box_format = bounding_box_format
        self.data_format = standardize_data_format(data_format)

    @property
    def image_size(self):
        """Settable tuple of `(height, width)` ints. The output image shape."""
        if self.resizing.height is None:
            return None
        return (self.resizing.height, self.resizing.width)

    @image_size.setter
    def image_size(self, value):
        if value is None:
            value = (None, None)
        self.resizing.height = value[0]
        self.resizing.width = value[1]

    @preprocessing_function
    def call(self, inputs):
        if self.image_size is not None:
            inputs = self.resizing(inputs)
        # Allow dictionary input for handling bounding boxes.
        if isinstance(inputs, dict):
            x = inputs["images"]
        else:
            x = inputs
        if self.scale is not None:
            x = x * self._expand_non_channel_dims(self.scale, x)
        if self.offset is not None:
            x = x + self._expand_non_channel_dims(self.offset, x)
        if isinstance(inputs, dict):
            inputs["images"] = x
        else:
            inputs = x
        return inputs

    def _expand_non_channel_dims(self, value, inputs):
        unbatched = len(ops.shape(inputs)) == 3
        channels_first = self.data_format == "channels_first"
        if unbatched:
            broadcast_dims = (1, 2) if channels_first else (0, 1)
        else:
            broadcast_dims = (0, 2, 3) if channels_first else (0, 1, 2)
        # If inputs are not a tensor type, return a numpy array.
        # This might happen when running under tf.data.
        if ops.is_tensor(inputs):
            # preprocessing decorator moves tensors to cpu in torch backend and
            # processed on CPU, and then converted back to the appropriate
            # device (potentially GPU) after preprocessing.
            if keras.backend.backend() == "torch" and self.image_size is None:
                return ops.expand_dims(value, broadcast_dims).cpu()
            return ops.expand_dims(value, broadcast_dims)
        else:
            return np.expand_dims(value, broadcast_dims)

    def get_config(self):
        config = super().get_config()
        config.update(
            {
                "image_size": self.image_size,
                "scale": self.scale,
                "offset": self.offset,
                "interpolation": self.interpolation,
                "crop_to_aspect_ratio": self.crop_to_aspect_ratio,
                "pad_to_aspect_ratio": self.pad_to_aspect_ratio,
                "bounding_box_format": self.bounding_box_format,
            }
        )
        return config

    @classproperty
    def presets(cls):
        """List built-in presets for an `ImageConverter` subclass."""
        return builtin_presets(cls)

    @classmethod
    def from_preset(
        cls,
        preset,
        **kwargs,
    ):
        """Instantiate a `keras_hub.layers.ImageConverter` from a model preset.

        A preset is a directory of configs, weights and other file assets used
        to save and load a pre-trained model. The `preset` can be passed as
        one of:

        1. a built-in preset identifier like `'pali_gemma_3b_224'`
        2. a Kaggle Models handle like
           `'kaggle://user/paligemma/keras/pali_gemma_3b_224'`
        3. a Hugging Face handle like `'hf://user/pali_gemma_3b_224'`
        4. a path to a local preset directory like `'./pali_gemma_3b_224'`

        You can run `cls.presets.keys()` to list all built-in presets available
        on the class.

        Args:
            preset: string. A built-in preset identifier, a Kaggle Models
                handle, a Hugging Face handle, or a path to a local directory.
            load_weights: bool. If `True`, the weights will be loaded into the
                model architecture. If `False`, the weights will be randomly
                initialized.

        Examples:
        ```python
        batch = np.random.randint(0, 256, size=(2, 512, 512, 3))

        # Resize images for `"pali_gemma_3b_224"`.
        converter = keras_hub.layers.ImageConverter.from_preset(
            "pali_gemma_3b_224"
        )
        converter(batch) # Output shape: (2, 224, 224, 3)

        # Resize images for `"pali_gemma_3b_448"` without cropping.
        converter = keras_hub.layers.ImageConverter.from_preset(
            "pali_gemma_3b_448",
            crop_to_aspect_ratio=False,
        )
        converter(batch) # Output shape: (2, 448, 448, 3)
        ```
        """
        loader = get_preset_loader(preset)
        backbone_cls = loader.check_backbone_class()
        if cls.backbone_cls != backbone_cls:
            cls = find_subclass(preset, cls, backbone_cls)
        return loader.load_image_converter(cls, **kwargs)

    def save_to_preset(self, preset_dir):
        """Save image converter to a preset directory.

        Args:
            preset_dir: The path to the local model preset directory.
        """
        saver = get_preset_saver(preset_dir)
        saver.save_image_converter(self)



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.basnet.basnet_backbone import BASNetBackbone


@keras_hub_export("keras_hub.layers.BASNetImageConverter")
class BASNetImageConverter(ImageConverter):
    backbone_cls = BASNetBackbone



from keras_hub.src.api_export import keras_hub_export
from keras_hub.src.layers.preprocessing.image_converter import ImageConverter
from keras_hub.src.models.retinanet.retinanet_backbone import RetinaNetBackbone


@keras_hub_export("keras_hub.layers.RetinaNetImageConverter")
class RetinaNetImageConverter(ImageConverter):
    backbone_cls = RetinaNetBackbone

    def __init__(
        self,
        *args,
        **kwargs,
    ):
        # TODO: update presets and remove these old config options. They were
        # never needed.
        if "norm_mean" in kwargs:
            kwargs["offset"] = [-x for x in kwargs.pop("norm_mean")]
        if "norm_std" in kwargs:
            kwargs["scale"] = [1.0 / x for x in kwargs.pop("norm_std")]
        super().__init__(*args, **kwargs)
